"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

from collections import abc as abc
from google.protobuf import descriptor as descriptor
from google.protobuf import message as message
from google.protobuf.internal import containers as containers
from google.protobuf.internal import enum_type_wrapper as enum_type_wrapper
import builtins as builtins
import sys
import typing as typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: descriptor.FileDescriptor

class _RestrictionVisibility:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: _TypeAlias = ValueType  # noqa: Y015

class _RestrictionVisibilityEnumTypeWrapper(enum_type_wrapper._EnumTypeWrapper[_RestrictionVisibility.ValueType], builtins.type):
    DESCRIPTOR: descriptor.EnumDescriptor
    NOT_VISIBLE: _RestrictionVisibility.ValueType  # 0
    PARTIALLY_VISIBLE: _RestrictionVisibility.ValueType  # 1
    FULLY_VISIBLE: _RestrictionVisibility.ValueType  # 2

class RestrictionVisibility(_RestrictionVisibility, metaclass=_RestrictionVisibilityEnumTypeWrapper):
    """Information about high-profile, military or government aircraft like
    Air Force One are not displayed. Other restricted aircraft may be anonymised,
    appearing as `BLOCKED` or `NA`.
    See [FAA's Limited Data Displayed](https://www.faa.gov/pilots/ladd) and
    [PIA program](https://www.faa.gov/air_traffic/technology/equipadsb/privacy)
    for more details.
    """

NOT_VISIBLE: RestrictionVisibility.ValueType  # 0
PARTIALLY_VISIBLE: RestrictionVisibility.ValueType  # 1
FULLY_VISIBLE: RestrictionVisibility.ValueType  # 2

class _Service:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: _TypeAlias = ValueType  # noqa: Y015

class _ServiceEnumTypeWrapper(enum_type_wrapper._EnumTypeWrapper[_Service.ValueType], builtins.type):
    DESCRIPTOR: descriptor.EnumDescriptor
    PASSENGER: _Service.ValueType  # 0
    CARGO: _Service.ValueType  # 1
    MILITARY_AND_GOVERNMENT: _Service.ValueType  # 2
    BUSINESS_JETS: _Service.ValueType  # 3
    GENERAL_AVIATION: _Service.ValueType  # 4
    HELICOPTERS: _Service.ValueType  # 5
    LIGHTER_THAN_AIR: _Service.ValueType  # 6
    GLIDERS: _Service.ValueType  # 7
    DRONES: _Service.ValueType  # 8
    GROUND_VEHICLES: _Service.ValueType  # 9
    OTHER_SERVICE: _Service.ValueType  # 10
    NON_CATEGORIZED: _Service.ValueType  # 11

class Service(_Service, metaclass=_ServiceEnumTypeWrapper): ...

PASSENGER: Service.ValueType  # 0
CARGO: Service.ValueType  # 1
MILITARY_AND_GOVERNMENT: Service.ValueType  # 2
BUSINESS_JETS: Service.ValueType  # 3
GENERAL_AVIATION: Service.ValueType  # 4
HELICOPTERS: Service.ValueType  # 5
LIGHTER_THAN_AIR: Service.ValueType  # 6
GLIDERS: Service.ValueType  # 7
DRONES: Service.ValueType  # 8
GROUND_VEHICLES: Service.ValueType  # 9
OTHER_SERVICE: Service.ValueType  # 10
NON_CATEGORIZED: Service.ValueType  # 11

class _TrafficType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: _TypeAlias = ValueType  # noqa: Y015

class _TrafficTypeEnumTypeWrapper(enum_type_wrapper._EnumTypeWrapper[_TrafficType.ValueType], builtins.type):
    DESCRIPTOR: descriptor.EnumDescriptor
    NONE: _TrafficType.ValueType  # 0
    GROUND_ONLY: _TrafficType.ValueType  # 1
    AIRBORNE_ONLY: _TrafficType.ValueType  # 2
    ALL: _TrafficType.ValueType  # 3

class TrafficType(_TrafficType, metaclass=_TrafficTypeEnumTypeWrapper): ...

NONE: TrafficType.ValueType  # 0
GROUND_ONLY: TrafficType.ValueType  # 1
AIRBORNE_ONLY: TrafficType.ValueType  # 2
ALL: TrafficType.ValueType  # 3

class _DataSource:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: _TypeAlias = ValueType  # noqa: Y015

class _DataSourceEnumTypeWrapper(enum_type_wrapper._EnumTypeWrapper[_DataSource.ValueType], builtins.type):
    DESCRIPTOR: descriptor.EnumDescriptor
    ADSB: _DataSource.ValueType  # 0
    MLAT: _DataSource.ValueType  # 1
    FLARM: _DataSource.ValueType  # 2
    FAA: _DataSource.ValueType  # 3
    ESTIMATED: _DataSource.ValueType  # 4
    SATELLITE: _DataSource.ValueType  # 5
    OTHER_DATA_SOURCE: _DataSource.ValueType  # 6
    UAT: _DataSource.ValueType  # 7
    SPIDERTRACKS: _DataSource.ValueType  # 8
    AUS: _DataSource.ValueType  # 9
    AIERON: _DataSource.ValueType  # 10

class DataSource(_DataSource, metaclass=_DataSourceEnumTypeWrapper): ...

ADSB: DataSource.ValueType  # 0
MLAT: DataSource.ValueType  # 1
FLARM: DataSource.ValueType  # 2
FAA: DataSource.ValueType  # 3
ESTIMATED: DataSource.ValueType  # 4
SATELLITE: DataSource.ValueType  # 5
OTHER_DATA_SOURCE: DataSource.ValueType  # 6
UAT: DataSource.ValueType  # 7
SPIDERTRACKS: DataSource.ValueType  # 8
AUS: DataSource.ValueType  # 9
AIERON: DataSource.ValueType  # 10

class _Icon:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: _TypeAlias = ValueType  # noqa: Y015

class _IconEnumTypeWrapper(enum_type_wrapper._EnumTypeWrapper[_Icon.ValueType], builtins.type):
    DESCRIPTOR: descriptor.EnumDescriptor
    B738: _Icon.ValueType  # 0
    """Boeing 737-800"""
    FGTR: _Icon.ValueType  # 1
    """Fighter jet"""
    ASW20: _Icon.ValueType  # 2
    """Glider ASW 20"""
    C206: _Icon.ValueType  # 3
    """Cessna 206"""
    C303: _Icon.ValueType  # 4
    """Cessna 303"""
    LJ60: _Icon.ValueType  # 5
    """Learjet 60"""
    Q300: _Icon.ValueType  # 6
    """Bombardier Dash 8 Q300"""
    B736: _Icon.ValueType  # 7
    """Boeing 737-600"""
    FOKKER100: _Icon.ValueType  # 8
    """Fokker 100"""
    RJ85: _Icon.ValueType  # 9
    """Avro RJ85"""
    A320: _Icon.ValueType  # 10
    """Airbus A320"""
    B757: _Icon.ValueType  # 11
    """Boeing 757"""
    B767: _Icon.ValueType  # 12
    """Boeing 767"""
    A3ST: _Icon.ValueType  # 13
    """Airbus A300-600ST Beluga"""
    MD11: _Icon.ValueType  # 14
    """McDonnell Douglas MD-11"""
    A330: _Icon.ValueType  # 15
    """Airbus A330"""
    A343: _Icon.ValueType  # 16
    """Airbus A340-300"""
    A346: _Icon.ValueType  # 17
    """Airbus A340-600"""
    B777: _Icon.ValueType  # 18
    """Boeing 777"""
    B747: _Icon.ValueType  # 19
    """Boeing 747"""
    A380: _Icon.ValueType  # 20
    """Airbus A380"""
    A225: _Icon.ValueType  # 21
    """Antonov An-225"""
    SI2: _Icon.ValueType  # 22
    """Solar Impulse 2"""
    EC: _Icon.ValueType  # 23
    """Eurocopter"""
    BALL: _Icon.ValueType  # 24
    """Balloon"""
    GRND: _Icon.ValueType  # 25
    """Ground vehicle"""
    SLEI: _Icon.ValueType  # 26
    """Santa Sleigh"""
    DRON: _Icon.ValueType  # 27
    """Drone"""
    SAT: _Icon.ValueType  # 28
    """Satellite"""
    ISS: _Icon.ValueType  # 29
    """International Space Station"""
    CONC: _Icon.ValueType  # 30
    """Concorde"""

class Icon(_Icon, metaclass=_IconEnumTypeWrapper): ...

B738: Icon.ValueType  # 0
"""Boeing 737-800"""
FGTR: Icon.ValueType  # 1
"""Fighter jet"""
ASW20: Icon.ValueType  # 2
"""Glider ASW 20"""
C206: Icon.ValueType  # 3
"""Cessna 206"""
C303: Icon.ValueType  # 4
"""Cessna 303"""
LJ60: Icon.ValueType  # 5
"""Learjet 60"""
Q300: Icon.ValueType  # 6
"""Bombardier Dash 8 Q300"""
B736: Icon.ValueType  # 7
"""Boeing 737-600"""
FOKKER100: Icon.ValueType  # 8
"""Fokker 100"""
RJ85: Icon.ValueType  # 9
"""Avro RJ85"""
A320: Icon.ValueType  # 10
"""Airbus A320"""
B757: Icon.ValueType  # 11
"""Boeing 757"""
B767: Icon.ValueType  # 12
"""Boeing 767"""
A3ST: Icon.ValueType  # 13
"""Airbus A300-600ST Beluga"""
MD11: Icon.ValueType  # 14
"""McDonnell Douglas MD-11"""
A330: Icon.ValueType  # 15
"""Airbus A330"""
A343: Icon.ValueType  # 16
"""Airbus A340-300"""
A346: Icon.ValueType  # 17
"""Airbus A340-600"""
B777: Icon.ValueType  # 18
"""Boeing 777"""
B747: Icon.ValueType  # 19
"""Boeing 747"""
A380: Icon.ValueType  # 20
"""Airbus A380"""
A225: Icon.ValueType  # 21
"""Antonov An-225"""
SI2: Icon.ValueType  # 22
"""Solar Impulse 2"""
EC: Icon.ValueType  # 23
"""Eurocopter"""
BALL: Icon.ValueType  # 24
"""Balloon"""
GRND: Icon.ValueType  # 25
"""Ground vehicle"""
SLEI: Icon.ValueType  # 26
"""Santa Sleigh"""
DRON: Icon.ValueType  # 27
"""Drone"""
SAT: Icon.ValueType  # 28
"""Satellite"""
ISS: Icon.ValueType  # 29
"""International Space Station"""
CONC: Icon.ValueType  # 30
"""Concorde"""

class _Status:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: _TypeAlias = ValueType  # noqa: Y015

class _StatusEnumTypeWrapper(enum_type_wrapper._EnumTypeWrapper[_Status.ValueType], builtins.type):
    DESCRIPTOR: descriptor.EnumDescriptor
    NORMAL: _Status.ValueType  # 0
    BACKGROUND: _Status.ValueType  # 1
    EMERGENCY: _Status.ValueType  # 2
    NOT_AVAILABLE: _Status.ValueType  # 3
    LIVE: _Status.ValueType  # 4

class Status(_Status, metaclass=_StatusEnumTypeWrapper): ...

NORMAL: Status.ValueType  # 0
BACKGROUND: Status.ValueType  # 1
EMERGENCY: Status.ValueType  # 2
NOT_AVAILABLE: Status.ValueType  # 3
LIVE: Status.ValueType  # 4

class _FlightStage:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: _TypeAlias = ValueType  # noqa: Y015

class _FlightStageEnumTypeWrapper(enum_type_wrapper._EnumTypeWrapper[_FlightStage.ValueType], builtins.type):
    DESCRIPTOR: descriptor.EnumDescriptor
    UNKNOWN: _FlightStage.ValueType  # 0
    ON_GROUND: _FlightStage.ValueType  # 1
    ASCENDING: _FlightStage.ValueType  # 2
    AIRBORNE: _FlightStage.ValueType  # 3
    DESCENDING: _FlightStage.ValueType  # 4
    DIVERSION: _FlightStage.ValueType  # 5

class FlightStage(_FlightStage, metaclass=_FlightStageEnumTypeWrapper):
    """used in FollowFlight, FlightDetails, PlaybackFlight"""

UNKNOWN: FlightStage.ValueType  # 0
ON_GROUND: FlightStage.ValueType  # 1
ASCENDING: FlightStage.ValueType  # 2
AIRBORNE: FlightStage.ValueType  # 3
DESCENDING: FlightStage.ValueType  # 4
DIVERSION: FlightStage.ValueType  # 5

class _DelayStatus:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: _TypeAlias = ValueType  # noqa: Y015

class _DelayStatusEnumTypeWrapper(enum_type_wrapper._EnumTypeWrapper[_DelayStatus.ValueType], builtins.type):
    DESCRIPTOR: descriptor.EnumDescriptor
    GRAY: _DelayStatus.ValueType  # 0
    GREEN: _DelayStatus.ValueType  # 1
    YELLOW: _DelayStatus.ValueType  # 2
    RED: _DelayStatus.ValueType  # 3

class DelayStatus(_DelayStatus, metaclass=_DelayStatusEnumTypeWrapper): ...

GRAY: DelayStatus.ValueType  # 0
GREEN: DelayStatus.ValueType  # 1
YELLOW: DelayStatus.ValueType  # 2
RED: DelayStatus.ValueType  # 3

class _EmergencyStatus:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: _TypeAlias = ValueType  # noqa: Y015

class _EmergencyStatusEnumTypeWrapper(enum_type_wrapper._EnumTypeWrapper[_EmergencyStatus.ValueType], builtins.type):
    DESCRIPTOR: descriptor.EnumDescriptor
    NO_EMERGENCY: _EmergencyStatus.ValueType  # 0
    GENERAL_EMERGENCY: _EmergencyStatus.ValueType  # 1
    LIFEGUARD_MEDICAL_EMERGENCY: _EmergencyStatus.ValueType  # 2
    MINIMUM_FUEL: _EmergencyStatus.ValueType  # 3
    NO_COMMUNICATIONS: _EmergencyStatus.ValueType  # 4
    UNLAWFUL_INTERFERENCE: _EmergencyStatus.ValueType  # 5
    DOWNED_AIRCRAFT: _EmergencyStatus.ValueType  # 6
    RESERVED: _EmergencyStatus.ValueType  # 7

class EmergencyStatus(_EmergencyStatus, metaclass=_EmergencyStatusEnumTypeWrapper):
    """NOTE: unused anywhere"""

NO_EMERGENCY: EmergencyStatus.ValueType  # 0
GENERAL_EMERGENCY: EmergencyStatus.ValueType  # 1
LIFEGUARD_MEDICAL_EMERGENCY: EmergencyStatus.ValueType  # 2
MINIMUM_FUEL: EmergencyStatus.ValueType  # 3
NO_COMMUNICATIONS: EmergencyStatus.ValueType  # 4
UNLAWFUL_INTERFERENCE: EmergencyStatus.ValueType  # 5
DOWNED_AIRCRAFT: EmergencyStatus.ValueType  # 6
RESERVED: EmergencyStatus.ValueType  # 7

@typing.final
class EMSInfo(message.Message):
    """Enhanced Mode-S data (EMS)"""

    DESCRIPTOR: descriptor.Descriptor

    QNH_FIELD_NUMBER: builtins.int
    AMCP_FIELD_NUMBER: builtins.int
    AFMS_FIELD_NUMBER: builtins.int
    OAT_FIELD_NUMBER: builtins.int
    IAS_FIELD_NUMBER: builtins.int
    TAS_FIELD_NUMBER: builtins.int
    MACH_FIELD_NUMBER: builtins.int
    AGPS_FIELD_NUMBER: builtins.int
    AGPSDIFF_FIELD_NUMBER: builtins.int
    APFLAGS_FIELD_NUMBER: builtins.int
    WIND_DIR_FIELD_NUMBER: builtins.int
    WIND_SPEED_FIELD_NUMBER: builtins.int
    RS_FIELD_NUMBER: builtins.int
    qnh: builtins.int
    """Altimeter setting (QFE or QNH/QNE)"""
    amcp: builtins.int
    """MCP/FCU selected altitude (BDS4,0)"""
    afms: builtins.int
    """FMS selected altitude (BDS4,0)"""
    oat: builtins.int
    """Outside air temperature, [TODO: update units]"""
    ias: builtins.int
    """Indicated airspeed (BDS6,0), knots"""
    tas: builtins.int
    """True airspeed (BDS0,5), knots"""
    mach: builtins.int
    """Mach number (BDS6,0), 1e3, e.g., `704` = M0.704"""
    agps: builtins.int
    """GPS altitude"""
    agpsdiff: builtins.int
    """Difference between amcp and agps"""
    apflags: builtins.int
    """Autopilot flags, often null"""
    wind_dir: builtins.int
    """Wind direction, [TODO: update units]"""
    wind_speed: builtins.int
    """Wind speed, [TODO: update units]"""
    rs: builtins.int
    def __init__(
        self,
        *,
        qnh: builtins.int = ...,
        amcp: builtins.int = ...,
        afms: builtins.int = ...,
        oat: builtins.int = ...,
        ias: builtins.int = ...,
        tas: builtins.int = ...,
        mach: builtins.int = ...,
        agps: builtins.int = ...,
        agpsdiff: builtins.int = ...,
        apflags: builtins.int = ...,
        wind_dir: builtins.int = ...,
        wind_speed: builtins.int = ...,
        rs: builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["afms", b"afms", "agps", b"agps", "agpsdiff", b"agpsdiff", "amcp", b"amcp", "apflags", b"apflags", "ias", b"ias", "mach", b"mach", "oat", b"oat", "qnh", b"qnh", "rs", b"rs", "tas", b"tas", "wind_dir", b"wind_dir", "wind_speed", b"wind_speed"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class EMSAvailability(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    QNH_AVAILABILITY_FIELD_NUMBER: builtins.int
    AMCP_AVAILABILITY_FIELD_NUMBER: builtins.int
    AFMS_AVAILABILITY_FIELD_NUMBER: builtins.int
    OAT_AVAILABILITY_FIELD_NUMBER: builtins.int
    IAS_AVAILABILITY_FIELD_NUMBER: builtins.int
    TAS_AVAILABILITY_FIELD_NUMBER: builtins.int
    MACH_AVAILABILITY_FIELD_NUMBER: builtins.int
    AGPS_AVAILABILITY_FIELD_NUMBER: builtins.int
    AGPSDIFF_AVAILABILITY_FIELD_NUMBER: builtins.int
    APFLAGS_AVAILABILITY_FIELD_NUMBER: builtins.int
    WIND_DIR_AVAILABILITY_FIELD_NUMBER: builtins.int
    WIND_SPEED_AVAILABILITY_FIELD_NUMBER: builtins.int
    RS_AVAILABILITY_FIELD_NUMBER: builtins.int
    qnh_availability: builtins.bool
    amcp_availability: builtins.bool
    afms_availability: builtins.bool
    oat_availability: builtins.bool
    ias_availability: builtins.bool
    tas_availability: builtins.bool
    mach_availability: builtins.bool
    agps_availability: builtins.bool
    agpsdiff_availability: builtins.bool
    apflags_availability: builtins.bool
    wind_dir_availability: builtins.bool
    wind_speed_availability: builtins.bool
    rs_availability: builtins.bool
    def __init__(
        self,
        *,
        qnh_availability: builtins.bool = ...,
        amcp_availability: builtins.bool = ...,
        afms_availability: builtins.bool = ...,
        oat_availability: builtins.bool = ...,
        ias_availability: builtins.bool = ...,
        tas_availability: builtins.bool = ...,
        mach_availability: builtins.bool = ...,
        agps_availability: builtins.bool = ...,
        agpsdiff_availability: builtins.bool = ...,
        apflags_availability: builtins.bool = ...,
        wind_dir_availability: builtins.bool = ...,
        wind_speed_availability: builtins.bool = ...,
        rs_availability: builtins.bool = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["afms_availability", b"afms_availability", "agps_availability", b"agps_availability", "agpsdiff_availability", b"agpsdiff_availability", "amcp_availability", b"amcp_availability", "apflags_availability", b"apflags_availability", "ias_availability", b"ias_availability", "mach_availability", b"mach_availability", "oat_availability", b"oat_availability", "qnh_availability", b"qnh_availability", "rs_availability", b"rs_availability", "tas_availability", b"tas_availability", "wind_dir_availability", b"wind_dir_availability", "wind_speed_availability", b"wind_speed_availability"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class Schedule(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    STD_FIELD_NUMBER: builtins.int
    ETD_FIELD_NUMBER: builtins.int
    ATD_FIELD_NUMBER: builtins.int
    STA_FIELD_NUMBER: builtins.int
    ETA_FIELD_NUMBER: builtins.int
    ATA_FIELD_NUMBER: builtins.int
    PROGRESS_PCT_FIELD_NUMBER: builtins.int
    std: builtins.int
    """Scheduled Time of Departure, Unix timestamp in seconds"""
    etd: builtins.int
    """Estimated Time of Departure, Unix timestamp in seconds"""
    atd: builtins.int
    """Actual Time of Departure, Unix timestamp in seconds"""
    sta: builtins.int
    """Scheduled Time of Arrival, Unix timestamp in seconds"""
    eta: builtins.int
    """Estimated Time of Arrival, Unix timestamp in seconds"""
    ata: builtins.int
    """Actual Time of Arrival, Unix timestamp in seconds"""
    progress_pct: builtins.int
    """Progress percentage of the flight"""
    def __init__(
        self,
        *,
        std: builtins.int = ...,
        etd: builtins.int = ...,
        atd: builtins.int = ...,
        sta: builtins.int = ...,
        eta: builtins.int = ...,
        ata: builtins.int = ...,
        progress_pct: builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["ata", b"ata", "atd", b"atd", "eta", b"eta", "etd", b"etd", "progress_pct", b"progress_pct", "sta", b"sta", "std", b"std"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class Route(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    FROM_FIELD_NUMBER: builtins.int
    TO_FIELD_NUMBER: builtins.int
    DIVERTED_TO_FIELD_NUMBER: builtins.int
    to: builtins.str
    diverted_to: builtins.str
    def __init__(
        self,
        *,
        to: builtins.str = ...,
        diverted_to: builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["diverted_to", b"diverted_to", "from", b"from", "to", b"to"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class ExtraFlightInfo(message.Message):
    """NOTE: Attributes `operated_by_id` (15), `squawk_availability` (16),
    `vspeed_availability` (17), `airspace_availability` (18),
    `airspace_id` (19), `server_time_ms` (21)
    only available when the flight is selected.
    """

    DESCRIPTOR: descriptor.Descriptor

    FLIGHT_FIELD_NUMBER: builtins.int
    REG_FIELD_NUMBER: builtins.int
    ROUTE_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    SQUAWK_FIELD_NUMBER: builtins.int
    VSPEED_FIELD_NUMBER: builtins.int
    AGE_FIELD_NUMBER: builtins.int
    COUNTRY_OF_REG_FIELD_NUMBER: builtins.int
    SCHEDULE_FIELD_NUMBER: builtins.int
    LOGO_ID_FIELD_NUMBER: builtins.int
    AIRSPACE_FIELD_NUMBER: builtins.int
    EMS_INFO_FIELD_NUMBER: builtins.int
    EMS_AVAILABILITY_FIELD_NUMBER: builtins.int
    ICAO_ADDRESS_FIELD_NUMBER: builtins.int
    OPERATED_BY_ID_FIELD_NUMBER: builtins.int
    SQUAWK_AVAILABILITY_FIELD_NUMBER: builtins.int
    VSPEED_AVAILABILITY_FIELD_NUMBER: builtins.int
    AIRSPACE_AVAILABILITY_FIELD_NUMBER: builtins.int
    AIRSPACE_ID_FIELD_NUMBER: builtins.int
    flight: builtins.str
    """IATA Flight number, e.g. `CX8747`"""
    reg: builtins.str
    """Aircraft registration, e.g. `B-HUJ`"""
    type: builtins.str
    """Aircraft type code, e.g. `B744`"""
    squawk: builtins.int
    """Squawk code, in base-10. e.g., `20852` (converted from octal)"""
    vspeed: builtins.int
    """Vertical speed, feet per minute"""
    age: builtins.str
    """Aircraft age, years in string format, e.g., `"17 years"`"""
    country_of_reg: builtins.int
    """Country code of registration"""
    logo_id: builtins.int
    """Airline logo ID"""
    airspace: builtins.int
    """FIR airspace ID"""
    icao_address: builtins.int
    """ICAO 24-bit address of the aircraft (hex)"""
    operated_by_id: builtins.int
    """Airline ID of the operator"""
    squawk_availability: builtins.bool
    vspeed_availability: builtins.bool
    airspace_availability: builtins.bool
    airspace_id: builtins.str
    """Airspace ID"""
    @builtins.property
    def route(self) -> Route: ...
    @builtins.property
    def schedule(self) -> Schedule: ...
    @builtins.property
    def ems_info(self) -> EMSInfo: ...
    @builtins.property
    def ems_availability(self) -> EMSAvailability: ...
    def __init__(
        self,
        *,
        flight: builtins.str = ...,
        reg: builtins.str = ...,
        route: Route | None = ...,
        type: builtins.str = ...,
        squawk: builtins.int = ...,
        vspeed: builtins.int = ...,
        age: builtins.str = ...,
        country_of_reg: builtins.int = ...,
        schedule: Schedule | None = ...,
        logo_id: builtins.int = ...,
        airspace: builtins.int = ...,
        ems_info: EMSInfo | None = ...,
        ems_availability: EMSAvailability | None = ...,
        icao_address: builtins.int = ...,
        operated_by_id: builtins.int = ...,
        squawk_availability: builtins.bool = ...,
        vspeed_availability: builtins.bool = ...,
        airspace_availability: builtins.bool = ...,
        airspace_id: builtins.str = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = typing.Literal["ems_availability", b"ems_availability", "ems_info", b"ems_info", "route", b"route", "schedule", b"schedule"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["age", b"age", "airspace", b"airspace", "airspace_availability", b"airspace_availability", "airspace_id", b"airspace_id", "country_of_reg", b"country_of_reg", "ems_availability", b"ems_availability", "ems_info", b"ems_info", "flight", b"flight", "icao_address", b"icao_address", "logo_id", b"logo_id", "operated_by_id", b"operated_by_id", "reg", b"reg", "route", b"route", "schedule", b"schedule", "squawk", b"squawk", "squawk_availability", b"squawk_availability", "type", b"type", "vspeed", b"vspeed", "vspeed_availability", b"vspeed_availability"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class SourceStats(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    SOURCE_FIELD_NUMBER: builtins.int
    COUNT_FIELD_NUMBER: builtins.int
    source: DataSource.ValueType
    count: builtins.int
    """Number of flights recorded with this source"""
    def __init__(
        self,
        *,
        source: DataSource.ValueType = ...,
        count: builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["count", b"count", "source", b"source"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class Stats(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    TOTAL_LIST_FIELD_NUMBER: builtins.int
    @builtins.property
    def total_list(self) -> containers.RepeatedCompositeFieldContainer[SourceStats]: ...
    def __init__(
        self,
        *,
        total_list: abc.Iterable[SourceStats] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["total_list", b"total_list"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class RecentPosition(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    DELTA_LAT_FIELD_NUMBER: builtins.int
    DELTA_LON_FIELD_NUMBER: builtins.int
    DELTA_MS_FIELD_NUMBER: builtins.int
    delta_lat: builtins.int
    """Offset from the current latitude, 1e5 degrees"""
    delta_lon: builtins.int
    """Offset from the current longitude, 1e5 degrees"""
    delta_ms: builtins.int
    """Delta time, milliseconds"""
    def __init__(
        self,
        *,
        delta_lat: builtins.int = ...,
        delta_lon: builtins.int = ...,
        delta_ms: builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["delta_lat", b"delta_lat", "delta_lon", b"delta_lon", "delta_ms", b"delta_ms"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class PositionBuffer(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    RECENT_POSITIONS_LIST_FIELD_NUMBER: builtins.int
    @builtins.property
    def recent_positions_list(self) -> containers.RepeatedCompositeFieldContainer[RecentPosition]:
        """List of recent positions"""

    def __init__(
        self,
        *,
        recent_positions_list: abc.Iterable[RecentPosition] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["recent_positions_list", b"recent_positions_list"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class Flight(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    FLIGHTID_FIELD_NUMBER: builtins.int
    LAT_FIELD_NUMBER: builtins.int
    LON_FIELD_NUMBER: builtins.int
    TRACK_FIELD_NUMBER: builtins.int
    ALT_FIELD_NUMBER: builtins.int
    SPEED_FIELD_NUMBER: builtins.int
    ICON_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    ON_GROUND_FIELD_NUMBER: builtins.int
    CALLSIGN_FIELD_NUMBER: builtins.int
    SOURCE_FIELD_NUMBER: builtins.int
    EXTRA_INFO_FIELD_NUMBER: builtins.int
    POSITION_BUFFER_FIELD_NUMBER: builtins.int
    TIMESTAMP_MS_FIELD_NUMBER: builtins.int
    flightid: builtins.int
    """FR24 flight id, in base-10 integer. Convert to hex for use in URLs."""
    lat: builtins.float
    """Latitude, degrees, -90 to 90"""
    lon: builtins.float
    """Longitude, degrees, -180 to 180"""
    track: builtins.int
    """True track angle, degrees clockwise from North"""
    alt: builtins.int
    """Barometric altitude, feet"""
    speed: builtins.int
    """Ground speed, knots"""
    icon: Icon.ValueType
    status: Status.ValueType
    timestamp: builtins.int
    """Last update timestamp, Unix timestamp in seconds"""
    on_ground: builtins.bool
    callsign: builtins.str
    """Callsign, e.g. `CPA8747`"""
    source: DataSource.ValueType
    timestamp_ms: builtins.int
    """Last update timestamp, milliseconds"""
    @builtins.property
    def extra_info(self) -> ExtraFlightInfo: ...
    @builtins.property
    def position_buffer(self) -> PositionBuffer: ...
    def __init__(
        self,
        *,
        flightid: builtins.int = ...,
        lat: builtins.float = ...,
        lon: builtins.float = ...,
        track: builtins.int = ...,
        alt: builtins.int = ...,
        speed: builtins.int = ...,
        icon: Icon.ValueType = ...,
        status: Status.ValueType = ...,
        timestamp: builtins.int = ...,
        on_ground: builtins.bool = ...,
        callsign: builtins.str = ...,
        source: DataSource.ValueType = ...,
        extra_info: ExtraFlightInfo | None = ...,
        position_buffer: PositionBuffer | None = ...,
        timestamp_ms: builtins.int = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = typing.Literal["extra_info", b"extra_info", "position_buffer", b"position_buffer"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["alt", b"alt", "callsign", b"callsign", "extra_info", b"extra_info", "flightid", b"flightid", "icon", b"icon", "lat", b"lat", "lon", b"lon", "on_ground", b"on_ground", "position_buffer", b"position_buffer", "source", b"source", "speed", b"speed", "status", b"status", "timestamp", b"timestamp", "timestamp_ms", b"timestamp_ms", "track", b"track"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class FlightProgress(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    TRAVERSED_DISTANCE_FIELD_NUMBER: builtins.int
    REMAINING_DISTANCE_FIELD_NUMBER: builtins.int
    ELAPSED_TIME_FIELD_NUMBER: builtins.int
    REMAINING_TIME_FIELD_NUMBER: builtins.int
    ETA_FIELD_NUMBER: builtins.int
    GREAT_CIRCLE_DISTANCE_FIELD_NUMBER: builtins.int
    MEAN_FLIGHT_TIME_FIELD_NUMBER: builtins.int
    FLIGHT_STAGE_FIELD_NUMBER: builtins.int
    DELAY_STATUS_FIELD_NUMBER: builtins.int
    PROGRESS_PCT_FIELD_NUMBER: builtins.int
    traversed_distance: builtins.int
    """Distance traversed in the flight, meters"""
    remaining_distance: builtins.int
    """Remaining distance to destination, meters"""
    elapsed_time: builtins.int
    """Elapsed flight time, seconds"""
    remaining_time: builtins.int
    """Remaining flight time, seconds"""
    eta: builtins.int
    """Estimated Time of Arrival, Unix timestamp in seconds"""
    great_circle_distance: builtins.int
    """Great circle distance of the first, meters"""
    mean_flight_time: builtins.int
    """Mean flight time for this route, seconds"""
    flight_stage: FlightStage.ValueType
    delay_status: DelayStatus.ValueType
    progress_pct: builtins.int
    """Flight progress percentage (e.g. `82`)"""
    def __init__(
        self,
        *,
        traversed_distance: builtins.int = ...,
        remaining_distance: builtins.int = ...,
        elapsed_time: builtins.int = ...,
        remaining_time: builtins.int = ...,
        eta: builtins.int = ...,
        great_circle_distance: builtins.int = ...,
        mean_flight_time: builtins.int = ...,
        flight_stage: FlightStage.ValueType = ...,
        delay_status: DelayStatus.ValueType = ...,
        progress_pct: builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["delay_status", b"delay_status", "elapsed_time", b"elapsed_time", "eta", b"eta", "flight_stage", b"flight_stage", "great_circle_distance", b"great_circle_distance", "mean_flight_time", b"mean_flight_time", "progress_pct", b"progress_pct", "remaining_distance", b"remaining_distance", "remaining_time", b"remaining_time", "traversed_distance", b"traversed_distance"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class ImageInfo(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    URL_FIELD_NUMBER: builtins.int
    COPYRIGHT_FIELD_NUMBER: builtins.int
    THUMBNAIL_FIELD_NUMBER: builtins.int
    MEDIUM_FIELD_NUMBER: builtins.int
    LARGE_FIELD_NUMBER: builtins.int
    SIDEVIEW_FIELD_NUMBER: builtins.int
    url: builtins.str
    """Image URL"""
    copyright: builtins.str
    """Copyright information (e.g. `"R Skywalker"`)"""
    thumbnail: builtins.str
    """URL for thumbnail image (e.g. `"https://cdn.jetphotos.com/200/5/459674_1738674373_tb.jpg"`)"""
    medium: builtins.str
    """URL for medium-sized image (e.g. `"https://cdn.jetphotos.com/400/5/459674_1738674373.jpg"`)"""
    large: builtins.str
    """URL for large-sized image (e.g. `"https://cdn.jetphotos.com/640/5/459674_1738674373.jpg"`)"""
    sideview: builtins.str
    def __init__(
        self,
        *,
        url: builtins.str = ...,
        copyright: builtins.str = ...,
        thumbnail: builtins.str = ...,
        medium: builtins.str = ...,
        large: builtins.str = ...,
        sideview: builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["copyright", b"copyright", "large", b"large", "medium", b"medium", "sideview", b"sideview", "thumbnail", b"thumbnail", "url", b"url"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class AircraftInfo(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    ICAO_ADDRESS_FIELD_NUMBER: builtins.int
    REG_FIELD_NUMBER: builtins.int
    COUNTRY_OF_REG_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    ICON_FIELD_NUMBER: builtins.int
    FULL_DESCRIPTION_FIELD_NUMBER: builtins.int
    MSN_FIELD_NUMBER: builtins.int
    SERVICE_FIELD_NUMBER: builtins.int
    AC_BIRTH_DATE_FIELD_NUMBER: builtins.int
    AC_AGE_FIELD_NUMBER: builtins.int
    IMAGES_LIST_FIELD_NUMBER: builtins.int
    IS_TEST_FLIGHT_FIELD_NUMBER: builtins.int
    MSN_AVAILABLE_FIELD_NUMBER: builtins.int
    AGE_AVAILABLE_FIELD_NUMBER: builtins.int
    REGISTERED_OWNERS_FIELD_NUMBER: builtins.int
    IS_COUNTRY_OF_REG_AVAILABLE_FIELD_NUMBER: builtins.int
    icao_address: builtins.int
    """ICAO 24-bit address of the aircraft"""
    reg: builtins.str
    """Registration number"""
    country_of_reg: builtins.int
    """Country of registration code (e.g. `3`)"""
    type: builtins.str
    """Aircraft type code (e.g. `"A359"`)"""
    icon: Icon.ValueType
    full_description: builtins.str
    """Full aircraft description (e.g. `"Airbus A350-941"`)"""
    msn: builtins.str
    service: Service.ValueType
    ac_birth_date: builtins.str
    """Aircraft birth date string (e.g. `"2017-06-28"`)"""
    ac_age: builtins.int
    """Aircraft age"""
    is_test_flight: builtins.bool
    msn_available: builtins.bool
    age_available: builtins.bool
    registered_owners: builtins.str
    """Aircraft owner name (e.g. `"Air France"`)"""
    is_country_of_reg_available: builtins.bool
    @builtins.property
    def images_list(self) -> containers.RepeatedCompositeFieldContainer[ImageInfo]: ...
    def __init__(
        self,
        *,
        icao_address: builtins.int = ...,
        reg: builtins.str = ...,
        country_of_reg: builtins.int = ...,
        type: builtins.str = ...,
        icon: Icon.ValueType = ...,
        full_description: builtins.str = ...,
        msn: builtins.str = ...,
        service: Service.ValueType = ...,
        ac_birth_date: builtins.str = ...,
        ac_age: builtins.int = ...,
        images_list: abc.Iterable[ImageInfo] | None = ...,
        is_test_flight: builtins.bool = ...,
        msn_available: builtins.bool = ...,
        age_available: builtins.bool = ...,
        registered_owners: builtins.str = ...,
        is_country_of_reg_available: builtins.bool = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["ac_age", b"ac_age", "ac_birth_date", b"ac_birth_date", "age_available", b"age_available", "country_of_reg", b"country_of_reg", "full_description", b"full_description", "icao_address", b"icao_address", "icon", b"icon", "images_list", b"images_list", "is_country_of_reg_available", b"is_country_of_reg_available", "is_test_flight", b"is_test_flight", "msn", b"msn", "msn_available", b"msn_available", "reg", b"reg", "registered_owners", b"registered_owners", "service", b"service", "type", b"type"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class Point(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    LATITUDE_FIELD_NUMBER: builtins.int
    LONGITUDE_FIELD_NUMBER: builtins.int
    latitude: builtins.int
    """Latitude, degrees, -90 to 90"""
    longitude: builtins.int
    """Longitude, degrees, -180 to 180"""
    def __init__(
        self,
        *,
        latitude: builtins.int = ...,
        longitude: builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["latitude", b"latitude", "longitude", b"longitude"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class Coordinate(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    CODE_FIELD_NUMBER: builtins.int
    POINT_FIELD_NUMBER: builtins.int
    code: builtins.str
    @builtins.property
    def point(self) -> Point: ...
    def __init__(
        self,
        *,
        code: builtins.str = ...,
        point: Point | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = typing.Literal["point", b"point"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["code", b"code", "point", b"point"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class Fix(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    AIRPORT_FIELD_NUMBER: builtins.int
    AREA_FIELD_NUMBER: builtins.int
    COORDINATE_FIELD_NUMBER: builtins.int
    airport: builtins.str
    area: builtins.str
    @builtins.property
    def coordinate(self) -> Coordinate: ...
    def __init__(
        self,
        *,
        airport: builtins.str = ...,
        area: builtins.str = ...,
        coordinate: Coordinate | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = typing.Literal["coordinate", b"coordinate"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["airport", b"airport", "area", b"area", "coordinate", b"coordinate"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class AltArrival(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    ARRIVAL_FIELD_NUMBER: builtins.int
    LENGTH_FIELD_NUMBER: builtins.int
    length: builtins.float
    @builtins.property
    def arrival(self) -> Fix: ...
    def __init__(
        self,
        *,
        arrival: Fix | None = ...,
        length: builtins.float = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = typing.Literal["arrival", b"arrival"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["arrival", b"arrival", "length", b"length"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class FlightPlan(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    DEPARTURE_FIELD_NUMBER: builtins.int
    DESTINATION_FIELD_NUMBER: builtins.int
    FLIGHT_PLAN_ICAO_FIELD_NUMBER: builtins.int
    LENGTH_FIELD_NUMBER: builtins.int
    ALT_ARRIVAL_1_FIELD_NUMBER: builtins.int
    ALT_ARRIVAL_2_FIELD_NUMBER: builtins.int
    WAYPOINTS_LIST_FIELD_NUMBER: builtins.int
    departure: builtins.str
    destination: builtins.str
    flight_plan_icao: builtins.str
    length: builtins.float
    @builtins.property
    def alt_arrival_1(self) -> AltArrival: ...
    @builtins.property
    def alt_arrival_2(self) -> AltArrival: ...
    @builtins.property
    def waypoints_list(self) -> containers.RepeatedCompositeFieldContainer[Point]: ...
    def __init__(
        self,
        *,
        departure: builtins.str = ...,
        destination: builtins.str = ...,
        flight_plan_icao: builtins.str = ...,
        length: builtins.float = ...,
        alt_arrival_1: AltArrival | None = ...,
        alt_arrival_2: AltArrival | None = ...,
        waypoints_list: abc.Iterable[Point] | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = typing.Literal["alt_arrival_1", b"alt_arrival_1", "alt_arrival_2", b"alt_arrival_2"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["alt_arrival_1", b"alt_arrival_1", "alt_arrival_2", b"alt_arrival_2", "departure", b"departure", "destination", b"destination", "flight_plan_icao", b"flight_plan_icao", "length", b"length", "waypoints_list", b"waypoints_list"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class ScheduleInfo(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    FLIGHT_NUMBER_FIELD_NUMBER: builtins.int
    OPERATED_BY_ID_FIELD_NUMBER: builtins.int
    PAINTED_AS_ID_FIELD_NUMBER: builtins.int
    ORIGIN_ID_FIELD_NUMBER: builtins.int
    DESTINATION_ID_FIELD_NUMBER: builtins.int
    DIVERTED_TO_ID_FIELD_NUMBER: builtins.int
    SCHEDULED_DEPARTURE_FIELD_NUMBER: builtins.int
    SCHEDULED_ARRIVAL_FIELD_NUMBER: builtins.int
    ACTUAL_DEPARTURE_FIELD_NUMBER: builtins.int
    ACTUAL_ARRIVAL_FIELD_NUMBER: builtins.int
    ARR_TERMINAL_FIELD_NUMBER: builtins.int
    ARR_GATE_FIELD_NUMBER: builtins.int
    BAGGAGE_BELT_FIELD_NUMBER: builtins.int
    flight_number: builtins.str
    """Flight number with airline prefix (e.g. `"AF334"`)"""
    operated_by_id: builtins.int
    """ID of the operating airline (e.g. `15`)"""
    painted_as_id: builtins.int
    """ID of the airline the aircraft is painted as (e.g. `15`)"""
    origin_id: builtins.int
    """Origin airport ID (e.g. `598`)"""
    destination_id: builtins.int
    """Destination airport ID (e.g. `451`)"""
    diverted_to_id: builtins.int
    """Diverted to airport ID"""
    scheduled_departure: builtins.int
    """Scheduled Time of Departure, Unix timestamp in seconds"""
    scheduled_arrival: builtins.int
    """Scheduled Time of Arrival, Unix timestamp in seconds"""
    actual_departure: builtins.int
    """Actual Time of Departure, Unix timestamp in seconds"""
    actual_arrival: builtins.int
    """Actual Time of Arrival, Unix timestamp in seconds"""
    arr_terminal: builtins.str
    """Arrival terminal (e.g. `"E"`)"""
    arr_gate: builtins.str
    """Arrival gate (e.g. `"E6"`)"""
    baggage_belt: builtins.str
    """Baggage belt (e.g. `"4"`)"""
    def __init__(
        self,
        *,
        flight_number: builtins.str = ...,
        operated_by_id: builtins.int = ...,
        painted_as_id: builtins.int = ...,
        origin_id: builtins.int = ...,
        destination_id: builtins.int = ...,
        diverted_to_id: builtins.int = ...,
        scheduled_departure: builtins.int = ...,
        scheduled_arrival: builtins.int = ...,
        actual_departure: builtins.int = ...,
        actual_arrival: builtins.int = ...,
        arr_terminal: builtins.str = ...,
        arr_gate: builtins.str = ...,
        baggage_belt: builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["actual_arrival", b"actual_arrival", "actual_departure", b"actual_departure", "arr_gate", b"arr_gate", "arr_terminal", b"arr_terminal", "baggage_belt", b"baggage_belt", "destination_id", b"destination_id", "diverted_to_id", b"diverted_to_id", "flight_number", b"flight_number", "operated_by_id", b"operated_by_id", "origin_id", b"origin_id", "painted_as_id", b"painted_as_id", "scheduled_arrival", b"scheduled_arrival", "scheduled_departure", b"scheduled_departure"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class ExtendedFlightInfo(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    FLIGHTID_FIELD_NUMBER: builtins.int
    LAT_FIELD_NUMBER: builtins.int
    LON_FIELD_NUMBER: builtins.int
    TRACK_FIELD_NUMBER: builtins.int
    ALT_FIELD_NUMBER: builtins.int
    SPEED_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    TIMESTAMP_MS_FIELD_NUMBER: builtins.int
    ON_GROUND_FIELD_NUMBER: builtins.int
    CALLSIGN_FIELD_NUMBER: builtins.int
    SOURCE_FIELD_NUMBER: builtins.int
    EMS_AVAILABILITY_FIELD_NUMBER: builtins.int
    EMS_INFO_FIELD_NUMBER: builtins.int
    SQUAWK_AVAILABILITY_FIELD_NUMBER: builtins.int
    SQUAWK_FIELD_NUMBER: builtins.int
    VSPEED_AVAILABILITY_FIELD_NUMBER: builtins.int
    VSPEED_FIELD_NUMBER: builtins.int
    AIRSPACE_AVAILABILITY_FIELD_NUMBER: builtins.int
    AIRSPACE_FIELD_NUMBER: builtins.int
    AIRSPACE_ID_FIELD_NUMBER: builtins.int
    SERVER_TIME_MS_FIELD_NUMBER: builtins.int
    flightid: builtins.int
    """Flight ID (e.g. `962788562` = `0x3962fcd2`)"""
    lat: builtins.float
    """Latitude, degrees, -90 to 90"""
    lon: builtins.float
    """Longitude, degrees, -180 to 180"""
    track: builtins.int
    """True track angle, degrees clockwise from North"""
    alt: builtins.int
    """Altitude, feet"""
    speed: builtins.int
    """Ground Speed, knots"""
    status: Status.ValueType
    timestamp_ms: builtins.int
    """Unix timestamp of message, milliseconds (e.g. `1741377144019`)"""
    on_ground: builtins.bool
    callsign: builtins.str
    """Callsign (e.g. `"AFR334"`)"""
    source: DataSource.ValueType
    squawk_availability: builtins.bool
    squawk: builtins.int
    """Squawk code, in base-10 (e.g. `3041` = 0o5741)"""
    vspeed_availability: builtins.bool
    vspeed: builtins.int
    """Vertical speed (e.g. `3328`)"""
    airspace_availability: builtins.bool
    airspace: builtins.str
    """Airspace, free-form text (e.g. `"Shannon UIR"`)"""
    airspace_id: builtins.str
    """Airspace ID (e.g. `"FIR_EINN_U"`)"""
    server_time_ms: builtins.int
    """Server timestamp, Unix timestamp in milliseconds (e.g. `1741377145974`)"""
    @builtins.property
    def ems_availability(self) -> EMSAvailability: ...
    @builtins.property
    def ems_info(self) -> EMSInfo: ...
    def __init__(
        self,
        *,
        flightid: builtins.int = ...,
        lat: builtins.float = ...,
        lon: builtins.float = ...,
        track: builtins.int = ...,
        alt: builtins.int = ...,
        speed: builtins.int = ...,
        status: Status.ValueType = ...,
        timestamp_ms: builtins.int = ...,
        on_ground: builtins.bool = ...,
        callsign: builtins.str = ...,
        source: DataSource.ValueType = ...,
        ems_availability: EMSAvailability | None = ...,
        ems_info: EMSInfo | None = ...,
        squawk_availability: builtins.bool = ...,
        squawk: builtins.int = ...,
        vspeed_availability: builtins.bool = ...,
        vspeed: builtins.int = ...,
        airspace_availability: builtins.bool = ...,
        airspace: builtins.str = ...,
        airspace_id: builtins.str = ...,
        server_time_ms: builtins.int = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = typing.Literal["ems_availability", b"ems_availability", "ems_info", b"ems_info"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["airspace", b"airspace", "airspace_availability", b"airspace_availability", "airspace_id", b"airspace_id", "alt", b"alt", "callsign", b"callsign", "ems_availability", b"ems_availability", "ems_info", b"ems_info", "flightid", b"flightid", "lat", b"lat", "lon", b"lon", "on_ground", b"on_ground", "server_time_ms", b"server_time_ms", "source", b"source", "speed", b"speed", "squawk", b"squawk", "squawk_availability", b"squawk_availability", "status", b"status", "timestamp_ms", b"timestamp_ms", "track", b"track", "vspeed", b"vspeed", "vspeed_availability", b"vspeed_availability"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class TrailPoint(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    SNAPSHOT_ID_FIELD_NUMBER: builtins.int
    LAT_FIELD_NUMBER: builtins.int
    LON_FIELD_NUMBER: builtins.int
    ALTITUDE_FIELD_NUMBER: builtins.int
    SPD_FIELD_NUMBER: builtins.int
    HEADING_FIELD_NUMBER: builtins.int
    VSPD_FIELD_NUMBER: builtins.int
    SOURCE_FIELD_NUMBER: builtins.int
    snapshot_id: builtins.int
    """Snapshot ID, likely Unix timestamp in seconds (e.g. `1741356300`)"""
    lat: builtins.float
    """Latitude, degrees, -90 to 90"""
    lon: builtins.float
    """Longitude, degrees, -180 to 180"""
    altitude: builtins.int
    """Altitude, feet"""
    spd: builtins.int
    """Ground speed, knots"""
    heading: builtins.int
    """True track angle, degrees clockwise from North.
    Note: despite the name, heading is not transmitted in ADS-B.
    """
    vspd: builtins.int
    """Vertical Speed, feet per minute"""
    source: DataSource.ValueType
    def __init__(
        self,
        *,
        snapshot_id: builtins.int = ...,
        lat: builtins.float = ...,
        lon: builtins.float = ...,
        altitude: builtins.int = ...,
        spd: builtins.int = ...,
        heading: builtins.int = ...,
        vspd: builtins.int = ...,
        source: DataSource.ValueType = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["altitude", b"altitude", "heading", b"heading", "lat", b"lat", "lon", b"lon", "snapshot_id", b"snapshot_id", "source", b"source", "spd", b"spd", "vspd", b"vspd"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class Duration(message.Message):
    """used in CountDown service"""

    DESCRIPTOR: descriptor.Descriptor

    COUNT_FIELD_NUMBER: builtins.int
    count: builtins.int
    def __init__(
        self,
        *,
        count: builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["count", b"count"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class Tick(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    COUNT_FIELD_NUMBER: builtins.int
    count: builtins.int
    def __init__(
        self,
        *,
        count: builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["count", b"count"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class RadarHistoryRecord(message.Message):
    """used in *trail services"""

    DESCRIPTOR: descriptor.Descriptor

    TIMESTAMP_FIELD_NUMBER: builtins.int
    LAT_FIELD_NUMBER: builtins.int
    LON_FIELD_NUMBER: builtins.int
    ALTITUDE_FIELD_NUMBER: builtins.int
    SPD_FIELD_NUMBER: builtins.int
    HEADING_FIELD_NUMBER: builtins.int
    VSPD_FIELD_NUMBER: builtins.int
    SQUAWK_FIELD_NUMBER: builtins.int
    SOURCE_FIELD_NUMBER: builtins.int
    CALLSIGN_FIELD_NUMBER: builtins.int
    timestamp: builtins.int
    """Observation timestamp, Unix timestamp in seconds"""
    lat: builtins.float
    """Latitude, degrees, -90 to 90"""
    lon: builtins.float
    """Longitude, degrees, -180 to 180"""
    altitude: builtins.int
    """Altitude, feet"""
    spd: builtins.int
    """Ground Speed, knots"""
    heading: builtins.int
    """True track angle, degrees clockwise from North.
    Note: despite the name, heading is not transmitted in ADS-B.
    """
    vspd: builtins.int
    """Vertical speed, feet per minute"""
    squawk: builtins.int
    source: DataSource.ValueType
    callsign: builtins.str
    """Callsign, e.g. `CPA959`"""
    def __init__(
        self,
        *,
        timestamp: builtins.int = ...,
        lat: builtins.float = ...,
        lon: builtins.float = ...,
        altitude: builtins.int = ...,
        spd: builtins.int = ...,
        heading: builtins.int = ...,
        vspd: builtins.int = ...,
        squawk: builtins.int = ...,
        source: DataSource.ValueType = ...,
        callsign: builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["altitude", b"altitude", "callsign", b"callsign", "heading", b"heading", "lat", b"lat", "lon", b"lon", "source", b"source", "spd", b"spd", "squawk", b"squawk", "timestamp", b"timestamp", "vspd", b"vspd"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...



#!/usr/bin/env -S uv run --script
# /// script
# requires-python = ">=3.10"
# dependencies = [
#     "griffe>=1.14.0",
#     "typing-extensions",
# ]
# ///
from __future__ import annotations

import ast
import logging
import sys
from pathlib import Path
from typing import Any, Generator

import griffe
from griffe._internal.extensions.dataclasses import _set_dataclass_init
from typing_extensions import TypeAlias

logger = griffe.get_logger(__name__)

DataclassPath: TypeAlias = str
"""The canonical path of a dataclass."""


class FR24CheckSignatureExtension(griffe.Extension):
    def __init__(
        self,
        decorator_path: str = "fr24.utils.static_check_signature",
    ) -> None:
        super().__init__()
        self.decorator_path = decorator_path
        self.methods_to_check: dict[griffe.Function, DataclassPath] = {}
        self.method_signature_issues = 0

    def on_function_instance(
        self,
        *,
        node: ast.AST | griffe.ObjectNode,
        func: griffe.Function,
        agent: griffe.Visitor | griffe.Inspector,
        **kwargs: Any,
    ) -> None:
        dataclass_path: DataclassPath | None = None
        for decorator in func.decorators:
            if decorator.callable_path == self.decorator_path:
                args = decorator.value
                assert isinstance(args, griffe.ExprCall)
                dataclass_name = args.arguments[0]
                assert (
                    isinstance(dataclass_name, griffe.ExprName)
                    and len(args.arguments) == 1
                )
                dataclass_path = dataclass_name.canonical_path
                break
        if dataclass_path is not None:
            self.methods_to_check[func] = dataclass_path

    def on_package(
        self, *, pkg: griffe.Module, loader: griffe.GriffeLoader, **kwargs: Any
    ) -> None:
        self.method_signature_issues = 0
        for func, dataclass_path in self.methods_to_check.items():
            self.method_signature_issues += len(
                collect_issues(loader.modules_collection, func, dataclass_path)
            )


def cleandoc(text: str) -> str:
    # similar to inspect.cleandoc, but more aggressive
    return " ".join(part.strip() for part in text.split("\n"))


def collect_issues(
    modules_collection: griffe.ModulesCollection,
    method: griffe.Function,
    dataclass_path: DataclassPath,
) -> list[str]:
    method_params = method.parameters
    spec_dataclass = modules_collection[dataclass_path]
    _set_dataclass_init(spec_dataclass)
    param_docs = {}
    if method.docstring:
        parsed_sections = method.docstring.parse(griffe.Parser.sphinx)
        for section in parsed_sections:
            if section.kind == griffe.DocstringSectionKind.parameters:
                for param in section.value:
                    param_docs[param.name] = cleandoc(param.description)
    issues = []
    klass_init: griffe.Function = spec_dataclass["__init__"]
    dataclass_params = klass_init.parameters._params
    for i, (method_arg, dataclass_field) in enumerate(
        zip(method_params, dataclass_params)
    ):
        if method_arg != dataclass_field:
            issues.append(
                f"argument {i} `{method_arg.name}`: name or kind mismatch\n"
                f"    expected `{dataclass_field}`\n"
                f"         got `{method_arg}`"
            )

        method_param_doc = param_docs.get(method_arg.name, None)
        dataclass_field_doc = (
            cleandoc(d.value) if (d := dataclass_field.docstring) else None
        )
        if method_param_doc != dataclass_field_doc:
            issues.append(
                f"argument {i} `{method_arg.name}`: docstring mismatch\n"
                f"    expected `{dataclass_field_doc}`\n"
                f"         got `{method_param_doc}`"
            )
    if issues:
        path = f"({method.relative_filepath}:{method.lineno})"
        s = "s" if len(issues) > 1 else ""
        err_str = (
            f"{len(issues)} issue{s} in `{method.canonical_path}` {path}:\n"
            + "\n".join(f"  - {issue}" for issue in issues)
            + "\n= help: update it to something like:\n```py\n"
            + "".join(gen_sphinx_signature(method, dataclass_params))
            + "\n```"
        )
        logger.warning(err_str)
    return issues


def gen_sphinx_signature(
    method: griffe.Function, dataclass_parameters: list[griffe.Parameter]
) -> Generator[str, None, None]:
    if "async" in method.labels:
        yield "async "
    yield f"def {method.path.split('.')[-1]}("
    n_args = len(dataclass_parameters)
    for i, field in enumerate(dataclass_parameters):
        yield field.name
        if field.annotation is not None:
            yield f": {field.annotation}"
        if field.default is not None:
            yield f" = {field.default}"
        if i < n_args - 1:
            yield ", "
    yield ")"
    if (return_annotation := method.returns) is not None:
        yield f" -> {return_annotation}"
    yield ":"

    doc_lines = []
    for param in dataclass_parameters:
        if param.docstring:
            doc_lines.append(f":param {param.name}: {param.docstring.value}")
    if doc_lines:
        yield '\n    """'
        for doc_line in doc_lines:
            yield f"\n    {doc_line}"
        yield '\n    """'


def main() -> None:
    """Check for signature issues in service methods."""
    checker = FR24CheckSignatureExtension()
    extensions = griffe.load_extensions(checker)
    griffe.load(
        "fr24",
        extensions=extensions,
        search_paths=[Path(__file__).parent.parent / "src"],
    )
    if err := checker.method_signature_issues:
        logger.error(f"found {err} signature issues{'s' if err > 1 else ''}")
        sys.exit(1)
    else:
        n_methods = len(checker.methods_to_check)
        logger.info(
            f"success: no signature issues found in {n_methods} methods."
        )


if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    main()



# ruff: noqa
# mypy: disable-error-code="top-level-await, no-redef"
# %%
# --8<-- [start:script]
import httpx

from fr24 import FR24
from fr24.proto.v1_pb2 import NearestFlightsResponse


async def get_nearest_flights(fr24: FR24) -> NearestFlightsResponse:
    nearest_result = await fr24.nearest_flights.fetch(
        lat=22.31257, lon=113.92708, radius=10000, limit=1500
    )
    return nearest_result.to_proto()


async def my_follow_flight() -> None:
    timeout = httpx.Timeout(5, read=120)
    async with FR24(client=httpx.AsyncClient(timeout=timeout)) as fr24:
        nearest_flights_result = await get_nearest_flights(fr24)
        flight_id = nearest_flights_result.flights_list[0].flight.flightid
        i = 0
        async for result in fr24.follow_flight.stream(flight_id=flight_id):
            print(f"##### {i} #####")
            print(result.to_proto())
            i += 1
            if i > 2:
                break


await my_follow_flight()
# --8<-- [end:script]
# %%
"""
# --8<-- [start:proto]
##### 0 #####
aircraft_info {
  icao_address: 7701252
  reg: "RP-C9903"
  type: "A321"
  icon: A320
  full_description: "Airbus A321-231"
  images_list {
    url: "https://www.jetphotos.com/photo/11825012"
    copyright: "yajoo"
    thumbnail: "https://cdn.jetphotos.com/200/5/2378185_1756401937_tb.jpg"
    medium: "https://cdn.jetphotos.com/400/5/2378185_1756401937.jpg"
    large: "https://cdn.jetphotos.com/640/5/2378185_1756401937.jpg"
  }
  images_list {
    url: "https://www.jetphotos.com/photo/11813525"
    copyright: "Landing HKG"
    thumbnail: "https://cdn.jetphotos.com/200/6/522343_1755503883_tb.jpg"
    medium: "https://cdn.jetphotos.com/400/6/522343_1755503883.jpg"
    large: "https://cdn.jetphotos.com/640/6/522343_1755503883.jpg"
  }
  images_list {
    url: "https://www.jetphotos.com/photo/11745327"
    copyright: "saTrL"
    thumbnail: "https://cdn.jetphotos.com/200/5/630999_1749204666_tb.jpg"
    medium: "https://cdn.jetphotos.com/400/5/630999_1749204666.jpg"
    large: "https://cdn.jetphotos.com/640/5/630999_1749204666.jpg"
  }
  msn_available: true
  age_available: true
  registered_owners: "Philippine Airlines"
  is_country_of_reg_available: true
}
schedule_info {
  flight_number: "PR311"
  operated_by_id: 98
  painted_as_id: 98
  origin_id: 1366
  destination_id: 2266
  scheduled_departure: 1758466800
  scheduled_arrival: 1758475200
  arr_terminal: "1"
}
flight_progress {
  traversed_distance: 657
  remaining_distance: 1143081
  great_circle_distance: 1143502
  mean_flight_time: 5886
  flight_stage: ON_GROUND
}
flight_info {
  flightid: 1011880908
  lat: 22.3098583
  lon: 113.922615
  track: 160
  speed: 8
  timestamp_ms: 1758466956660
  on_ground: true
  callsign: "PAL311"
  ems_availability {
    rs_availability: true
  }
  squawk_availability: true
  airspace_availability: true
  server_time_ms: 1758466960115
}
flight_trail_list {
  snapshot_id: 1758466272
  lat: 22.3157578
  lon: 113.932076
  spd: 3
  heading: 177
}
...
}
flight_trail_list {
  snapshot_id: 1758466950
  lat: 22.3100777
  lon: 113.922523
  spd: 9
  heading: 160
}
position_buffer {
  recent_positions_list {
    delta_lat: -5
    delta_lon: 2
    delta_ms: 1670
  }
  recent_positions_list {
    delta_lat: -9
    delta_lon: 3
    delta_ms: 2610
  }
  recent_positions_list {
    delta_lat: -21
    delta_lon: 9
    delta_ms: 6010
  }
  recent_positions_list {
    delta_lat: -47
    delta_lon: 18
    delta_ms: 12350
  }
}

##### 1 #####
schedule_info {
  flight_number: "PR311"
  operated_by_id: 98
  painted_as_id: 98
  origin_id: 1366
  destination_id: 2266
  scheduled_departure: 1758466800
  scheduled_arrival: 1758475200
  arr_terminal: "1"
}
flight_progress {
  traversed_distance: 662
  remaining_distance: 1143066
  great_circle_distance: 1143502
  mean_flight_time: 5886
  flight_stage: ON_GROUND
}
flight_info {
  flightid: 1011880908
  lat: 22.3097286
  lon: 113.922668
  track: 160
  speed: 8
  timestamp_ms: 1758466960150
  on_ground: true
  callsign: "PAL311"
  ems_availability {
    rs_availability: true
  }
  squawk_availability: true
  airspace_availability: true
  server_time_ms: 1758466963120
}

##### 2 #####
schedule_info {
  flight_number: "PR311"
  operated_by_id: 98
  painted_as_id: 98
  origin_id: 1366
  destination_id: 2266
  scheduled_departure: 1758466800
  scheduled_arrival: 1758475200
  arr_terminal: "1"
}
flight_progress {
  traversed_distance: 667
  remaining_distance: 1143051
  great_circle_distance: 1143502
  mean_flight_time: 5886
  flight_stage: ON_GROUND
}
flight_info {
  flightid: 1011880908
  lat: 22.3095818
  lon: 113.922714
  track: 160
  speed: 9
  timestamp_ms: 1758466963941
  on_ground: true
  callsign: "PAL311"
  ems_availability {
    rs_availability: true
  }
  squawk_availability: true
  airspace_availability: true
  server_time_ms: 1758466966122
}

# --8<-- [end:proto]
"""



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fr24/proto/_historic_trail.proto
# Protobuf Python Version: 6.33.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    33,
    0,
    '',
    'fr24/proto/_historic_trail.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fr24.proto import _common_pb2 as fr24_dot_proto_dot___common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n fr24/proto/_historic_trail.proto\x1a\x18\x66r24/proto/_common.proto\")\n\x14HistoricTrailRequest\x12\x11\n\tflight_id\x18\x01 \x01(\x07\"P\n\x15HistoricTrailResponse\x12\x37\n\x12radar_records_list\x18\x01 \x03(\x0b\x32\x1b._common.RadarHistoryRecordb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fr24.proto._historic_trail_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_HISTORICTRAILREQUEST']._serialized_start=62
  _globals['_HISTORICTRAILREQUEST']._serialized_end=103
  _globals['_HISTORICTRAILRESPONSE']._serialized_start=105
  _globals['_HISTORICTRAILRESPONSE']._serialized_end=185
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

from collections import abc as abc
from fr24.proto import _common_pb2 as _common_pb2
from google.protobuf import descriptor as descriptor
from google.protobuf import message as message
from google.protobuf.internal import containers as containers
import builtins as builtins
import sys
import typing as typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: descriptor.FileDescriptor

@typing.final
class Geolocation(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    LAT_FIELD_NUMBER: builtins.int
    LON_FIELD_NUMBER: builtins.int
    lat: builtins.float
    """Latitude, degrees, -90 to 90"""
    lon: builtins.float
    """Longitude, degrees, -180 to 180"""
    def __init__(
        self,
        *,
        lat: builtins.float = ...,
        lon: builtins.float = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["lat", b"lat", "lon", b"lon"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class NearestFlightsRequest(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    LOCATION_FIELD_NUMBER: builtins.int
    RADIUS_FIELD_NUMBER: builtins.int
    LIMIT_FIELD_NUMBER: builtins.int
    radius: builtins.int
    """Radius, metres"""
    limit: builtins.int
    """Maximum number of aircraft to return"""
    @builtins.property
    def location(self) -> Geolocation: ...
    def __init__(
        self,
        *,
        location: Geolocation | None = ...,
        radius: builtins.int = ...,
        limit: builtins.int = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = typing.Literal["location", b"location"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["limit", b"limit", "location", b"location", "radius", b"radius"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class NearbyFlight(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    FLIGHT_FIELD_NUMBER: builtins.int
    DISTANCE_FIELD_NUMBER: builtins.int
    distance: builtins.int
    """Distance from the location, metres"""
    @builtins.property
    def flight(self) -> _common_pb2.Flight: ...
    def __init__(
        self,
        *,
        flight: _common_pb2.Flight | None = ...,
        distance: builtins.int = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = typing.Literal["flight", b"flight"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["distance", b"distance", "flight", b"flight"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class NearestFlightsResponse(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    FLIGHTS_LIST_FIELD_NUMBER: builtins.int
    @builtins.property
    def flights_list(self) -> containers.RepeatedCompositeFieldContainer[NearbyFlight]: ...
    def __init__(
        self,
        *,
        flights_list: abc.Iterable[NearbyFlight] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["flights_list", b"flights_list"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...



import httpx
import pytest

from fr24 import FR24
from fr24.service import NearestFlightsResult


@pytest.mark.anyio
async def test_follow_flight(
    nearest_flights_result: NearestFlightsResult,
) -> None:
    flight_id = (
        nearest_flights_result.to_proto().flights_list[-1].flight.flightid
    )
    timeout = httpx.Timeout(5, read=360)
    async with FR24(client=httpx.AsyncClient(timeout=timeout)) as fr24:
        i = 0
        async for result in fr24.follow_flight.stream(flight_id=flight_id):
            if i == 0:
                assert len(result.to_proto().flight_trail_list)
            assert result.to_proto().flight_info.flightid == flight_id
            i += 1
            if i > 2:
                break



# ruff: noqa
# mypy: disable-error-code="top-level-await, no-redef"
# %%
# --8<-- [start:script]
from fr24 import FR24, FR24Cache
import polars as pl


async def get_last_flight(fr24: FR24, *, reg: str = "B-LRA") -> tuple[int, int]:
    flight_list_result = await fr24.flight_list.fetch(reg=reg)
    df = flight_list_result.to_polars()

    landed = df.filter(pl.col("status").str.starts_with("Landed"))
    assert landed.height > 0, "no landed flights found"

    flight_id = landed[0, "flight_id"]
    stod = int(landed[0, "ATOD"].timestamp())
    return flight_id, stod


async def my_playback_flight() -> None:
    async with FR24() as fr24:
        flight_id, timestamp = await get_last_flight(fr24)
        result = await fr24.playback_flight.fetch(
            flight_id=flight_id, timestamp=timestamp
        )
        print(result)
        print(result.to_dict())
        print(result.to_polars())
        result.write_table(FR24Cache.default())


await my_playback_flight()
# --8<-- [end:script]

# %%
"""
# --8<-- [start:result]
PlaybackFlightResult(
    request=PlaybackFlightParams(flight_id=1011667820, timestamp=1758386815),
    response=<Response [200 OK]>
)
# --8<-- [end:result]
# --8<-- [start:dict]
{
    "aircraft_info": {
        "icao_address": 7867035,
        "reg": "B-LRA",
        "type": "A359",
        "icon": "A330",
        "full_description": "Airbus A350-941",
        "images_list": [
            {
                "url": "https://www.jetphotos.com/photo/11806625",
                "copyright": "EthanLi",
                "thumbnail": "https://cdn.jetphotos.com/200/5/682006_1754851444_tb.jpg",
                "medium": "https://cdn.jetphotos.com/400/5/682006_1754851444.jpg",
                "large": "https://cdn.jetphotos.com/640/5/682006_1754851444.jpg",
            },
            {
                "url": "https://www.jetphotos.com/photo/11728089",
                "copyright": "Hypocrite",
                "thumbnail": "https://cdn.jetphotos.com/200/6/1954570_1747460006_tb.jpg",
                "medium": "https://cdn.jetphotos.com/400/6/1954570_1747460006.jpg",
                "large": "https://cdn.jetphotos.com/640/6/1954570_1747460006.jpg",
            },
            {
                "url": "https://www.jetphotos.com/photo/11721357",
                "copyright": "Steven-lzy",
                "thumbnail": "https://cdn.jetphotos.com/200/5/778073_1746782825_tb.jpg",
                "medium": "https://cdn.jetphotos.com/400/5/778073_1746782825.jpg",
                "large": "https://cdn.jetphotos.com/640/5/778073_1746782825.jpg",
            },
        ],
        "msn_available": True,
        "age_available": True,
        "registered_owners": "Cathay Pacific",
        "is_country_of_reg_available": True,
    },
    "schedule_info": {
        "flight_number": "CX170",
        "operated_by_id": 57,
        "painted_as_id": 57,
        "origin_id": 2730,
        "destination_id": 1366,
        "scheduled_departure": 1758384600,
        "scheduled_arrival": 1758412500,
        "actual_departure": 1758386815,
        "actual_arrival": 1758414073,
        "arr_terminal": "1",
        "baggage_belt": "14",
    },
    "flight_info": {
        "flightid": 1011667820,
        "lat": -31.94989,
        "lon": 115.962234,
        "track": 194,
        "speed": 168,
        "timestamp_ms": "1758386809081",
        "callsign": "CPA170",
        "ems_availability": {
            "qnh_availability": True,
            "amcp_availability": True,
            "mach_availability": True,
            "agps_availability": True,
            "agpsdiff_availability": True,
            "rs_availability": True,
        },
        "squawk_availability": True,
        "vspeed_availability": True,
        "airspace_availability": True,
        "server_time_ms": "1758466605103",
    },
    "flight_trail_list": [
        {
            "snapshot_id": "1758385543",
            "lat": -31.94054,
            "lon": 115.973206,
            "heading": 151,
        },
        ...
        {
            "snapshot_id": "1758415950",
            "lat": 22.31186,
            "lon": 113.92748,
            "spd": 1,
            "heading": 295,
        },
    ],
}
# --8<-- [end:dict]
# --8<-- [start:polars]
shape: (1, 24)
┌─────────────┬───────┬──────────┬─────────────┬───┬──────────┬────────┬─────────────┬─────────────┐
│ icao_addres ┆ reg   ┆ typecode ┆ flight_numb ┆ … ┆ callsign ┆ squawk ┆ ems         ┆ flight_trai │
│ s           ┆ ---   ┆ ---      ┆ er          ┆   ┆ ---      ┆ ---    ┆ ---         ┆ l_list      │
│ ---         ┆ str   ┆ str      ┆ ---         ┆   ┆ str      ┆ u16    ┆ struct[13]  ┆ ---         │
│ u32         ┆       ┆          ┆ str         ┆   ┆          ┆        ┆             ┆ list[struct │
│             ┆       ┆          ┆             ┆   ┆          ┆        ┆             ┆ [7]]        │
╞═════════════╪═══════╪══════════╪═════════════╪═══╪══════════╪════════╪═════════════╪═════════════╡
│ 7867035     ┆ B-LRA ┆ A359     ┆ CX758       ┆ … ┆ CPA758   ┆ 0      ┆ {0,0,0,0,0, ┆ [{174815111 │
│             ┆       ┆          ┆             ┆   ┆          ┆        ┆ 0,0,0,0,0,0 ┆ 3,1.341973, │
│             ┆       ┆          ┆             ┆   ┆          ┆        ┆ ,0,0}       ┆ 103.9865…   │
└─────────────┴───────┴──────────┴─────────────┴───┴──────────┴────────┴─────────────┴─────────────┘
# --8<-- [end:polars]
"""



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fr24/proto/_top_flights.proto
# Protobuf Python Version: 6.33.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    33,
    0,
    '',
    'fr24/proto/_top_flights.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1d\x66r24/proto/_top_flights.proto\"\xf7\x01\n\x0e\x46ollowedFlight\x12\x11\n\tflight_id\x18\x01 \x01(\r\x12\x13\n\x0blive_clicks\x18\x02 \x01(\r\x12\x14\n\x0ctotal_clicks\x18\x03 \x01(\r\x12\x15\n\rflight_number\x18\x04 \x01(\t\x12\x10\n\x08\x63\x61llsign\x18\x05 \x01(\t\x12\x0e\n\x06squawk\x18\x06 \x01(\r\x12\x11\n\tfrom_iata\x18\x07 \x01(\t\x12\x11\n\tfrom_city\x18\x08 \x01(\t\x12\x0f\n\x07to_iata\x18\t \x01(\t\x12\x0f\n\x07to_city\x18\n \x01(\t\x12\x0c\n\x04type\x18\x0b \x01(\t\x12\x18\n\x10\x66ull_description\x18\x0c \x01(\t\"\"\n\x11TopFlightsRequest\x12\r\n\x05limit\x18\x01 \x01(\r\">\n\x12TopFlightsResponse\x12(\n\x0fscoreboard_list\x18\x01 \x03(\x0b\x32\x0f.FollowedFlightb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fr24.proto._top_flights_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_FOLLOWEDFLIGHT']._serialized_start=34
  _globals['_FOLLOWEDFLIGHT']._serialized_end=281
  _globals['_TOPFLIGHTSREQUEST']._serialized_start=283
  _globals['_TOPFLIGHTSREQUEST']._serialized_end=317
  _globals['_TOPFLIGHTSRESPONSE']._serialized_start=319
  _globals['_TOPFLIGHTSRESPONSE']._serialized_end=381
# @@protoc_insertion_point(module_scope)



import time
from typing import Generator

import pytest

from fr24 import FR24
from fr24.service import NearestFlightsResult


@pytest.fixture(autouse=True, scope="function")
def slow_down() -> Generator[None, None, None]:
    """Ratelimit API tests to avoid overloading the server."""

    yield
    time.sleep(2.5)


@pytest.fixture
async def nearest_flights_result(
    fr24: FR24,
) -> NearestFlightsResult:
    LON = 113.92708
    LAT = 22.31257
    RADIUS = 10000
    LIMIT = 1500

    result = await fr24.nearest_flights.fetch(
        lon=LON, lat=LAT, radius=RADIUS, limit=LIMIT
    )
    return result



# ruff: noqa
# mypy: disable-error-code="top-level-await, no-redef"
# %%
# --8<-- [start:script0]
import httpx
from fr24.grpc import PlaybackFlightParams, playback_flight
from fr24.proto.v1_pb2 import PlaybackFlightResponse
from fr24.proto import parse_data
from fr24.proto.headers import get_grpc_headers


async def playback_flight_data() -> PlaybackFlightResponse:
    headers = httpx.Headers(get_grpc_headers(auth=None))
    async with httpx.AsyncClient() as client:
        message = PlaybackFlightParams(
            flight_id=0x3C500FDB, timestamp=1758467125
        )
        response = await playback_flight(client, message, headers)
        return parse_data(response.content, PlaybackFlightResponse).unwrap()


data = await playback_flight_data()
data
# --8<-- [end:script0]
# %%
"""
# --8<-- [start:output0]
aircraft_info {
  icao_address: 7866977
  reg: "B-KQM"
  type: "B77W"
  icon: B777
  full_description: "Boeing 777-367(ER)"
  images_list {
    url: "https://www.jetphotos.com/photo/11825817"
    copyright: "ZSHC_Linzx"
    thumbnail: "https://cdn.jetphotos.com/200/5/1291679_1756473618_tb.jpg"
    medium: "https://cdn.jetphotos.com/400/5/1291679_1756473618.jpg"
    large: "https://cdn.jetphotos.com/640/5/1291679_1756473618.jpg"
  }
  images_list {
    url: "https://www.jetphotos.com/photo/11813969"
    copyright: "CAN-Eric"
    thumbnail: "https://cdn.jetphotos.com/200/5/967682_1755532162_tb.jpg"
    medium: "https://cdn.jetphotos.com/400/5/967682_1755532162.jpg"
    large: "https://cdn.jetphotos.com/640/5/967682_1755532162.jpg"
  }
  images_list {
    url: "https://www.jetphotos.com/photo/11803561"
    copyright: "cc12214"
    thumbnail: "https://cdn.jetphotos.com/200/6/585221_1754572527_tb.jpg"
    medium: "https://cdn.jetphotos.com/400/6/585221_1754572527.jpg"
    large: "https://cdn.jetphotos.com/640/6/585221_1754572527.jpg"
  }
  msn_available: true
  age_available: true
  registered_owners: "Cathay Pacific (Oneworld Livery)"
  is_country_of_reg_available: true
}
schedule_info {
  flight_number: "CX251"
  operated_by_id: 57
  painted_as_id: 57
  origin_id: 1366
  destination_id: 1942
  scheduled_departure: 1758465600
  scheduled_arrival: 1758516000
  actual_departure: 1758466901
  arr_terminal: "3"
}
flight_info {
  flightid: 1011879899
  lat: 22.2923298
  lon: 114.087372
  track: 173
  alt: 6325
  speed: 242
  timestamp_ms: 1758467118710
  callsign: "CPA251"
  ems_availability {
    qnh_availability: true
    amcp_availability: true
    oat_availability: true
    ias_availability: true
    tas_availability: true
    mach_availability: true
    agps_availability: true
    agpsdiff_availability: true
    wind_dir_availability: true
    wind_speed_availability: true
    rs_availability: true
  }
  squawk_availability: true
  vspeed_availability: true
  airspace_availability: true
  server_time_ms: 1758467148386
}
flight_trail_list {
  snapshot_id: 1758465991
  lat: 22.3132782
  lon: 113.934616
  heading: 70
}
...
flight_trail_list {
  snapshot_id: 1758467147
  lat: 22.2578888
  lon: 114.087273
  altitude: 6800
  spd: 267
  heading: 180
}
"""



import pytest

from fr24 import FR24Cache
from fr24.service import NearestFlightsResult


@pytest.mark.anyio
async def test_nearest_flights_simple(
    nearest_flights_result: NearestFlightsResult,
) -> None:
    num_flights = len(nearest_flights_result.to_proto().flights_list)
    assert len(nearest_flights_result.to_proto().flights_list) > 2

    df = nearest_flights_result.to_polars()
    assert df.height == num_flights
    assert df.width == 19


@pytest.mark.anyio
async def test_nearest_flights_file_ops(
    nearest_flights_result: NearestFlightsResult, cache: FR24Cache
) -> None:
    lon = nearest_flights_result.request.lon
    lat = nearest_flights_result.request.lat
    timestamp = nearest_flights_result.timestamp
    fp = (
        cache.path
        / "nearest_flights"
        / f"{int(lon * 1e6)}_{int(lat * 1e6)}_{timestamp}.parquet"
    )
    fp.parent.mkdir(parents=True, exist_ok=True)
    fp.unlink(missing_ok=True)

    nearest_flights_result.write_table(cache)
    assert fp.exists(), (
        f"{fp} not in {list(f.name for f in fp.parent.glob('*'))}"
    )

    df_local = cache.nearest_flights.scan_table(lon, lat, timestamp).collect()
    assert df_local.equals(nearest_flights_result.to_polars())

    fp.unlink()



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

from collections import abc as abc
from fr24.proto import _common_pb2 as _common_pb2
from google.protobuf import descriptor as descriptor
from google.protobuf import field_mask_pb2 as field_mask_pb2
from google.protobuf import message as message
from google.protobuf.internal import containers as containers
from google.protobuf.internal import enum_type_wrapper as enum_type_wrapper
import builtins as builtins
import sys
import typing as typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: descriptor.FileDescriptor

class _AirportFilterType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: _TypeAlias = ValueType  # noqa: Y015

class _AirportFilterTypeEnumTypeWrapper(enum_type_wrapper._EnumTypeWrapper[_AirportFilterType.ValueType], builtins.type):
    DESCRIPTOR: descriptor.EnumDescriptor
    BOTH: _AirportFilterType.ValueType  # 0
    INBOUND: _AirportFilterType.ValueType  # 1
    OUTBOUND: _AirportFilterType.ValueType  # 2

class AirportFilterType(_AirportFilterType, metaclass=_AirportFilterTypeEnumTypeWrapper): ...

BOTH: AirportFilterType.ValueType  # 0
INBOUND: AirportFilterType.ValueType  # 1
OUTBOUND: AirportFilterType.ValueType  # 2

class _AirlineFilterType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: _TypeAlias = ValueType  # noqa: Y015

class _AirlineFilterTypeEnumTypeWrapper(enum_type_wrapper._EnumTypeWrapper[_AirlineFilterType.ValueType], builtins.type):
    DESCRIPTOR: descriptor.EnumDescriptor
    PAINTED_AS: _AirlineFilterType.ValueType  # 0
    OPERATED_BY: _AirlineFilterType.ValueType  # 1

class AirlineFilterType(_AirlineFilterType, metaclass=_AirlineFilterTypeEnumTypeWrapper): ...

PAINTED_AS: AirlineFilterType.ValueType  # 0
OPERATED_BY: AirlineFilterType.ValueType  # 1

@typing.final
class LocationBoundaries(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    NORTH_FIELD_NUMBER: builtins.int
    SOUTH_FIELD_NUMBER: builtins.int
    WEST_FIELD_NUMBER: builtins.int
    EAST_FIELD_NUMBER: builtins.int
    north: builtins.float
    """Maximum Latitude, degrees, -90 to 90"""
    south: builtins.float
    """Minimum Latitude, degrees, -90 to 90"""
    west: builtins.float
    """Minimum Longitude, degrees, -180 to 180"""
    east: builtins.float
    """Maximum Longitude, degrees, -180 to 180"""
    def __init__(
        self,
        *,
        north: builtins.float = ...,
        south: builtins.float = ...,
        west: builtins.float = ...,
        east: builtins.float = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["east", b"east", "north", b"north", "south", b"south", "west", b"west"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class VisibilitySettings(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    SOURCES_LIST_FIELD_NUMBER: builtins.int
    SERVICES_LIST_FIELD_NUMBER: builtins.int
    TRAFFIC_TYPE_FIELD_NUMBER: builtins.int
    ONLY_RESTRICTED_FIELD_NUMBER: builtins.int
    traffic_type: _common_pb2.TrafficType.ValueType
    only_restricted: builtins.bool
    @builtins.property
    def sources_list(self) -> containers.RepeatedScalarFieldContainer[_common_pb2.DataSource.ValueType]: ...
    @builtins.property
    def services_list(self) -> containers.RepeatedScalarFieldContainer[_common_pb2.Service.ValueType]: ...
    def __init__(
        self,
        *,
        sources_list: abc.Iterable[_common_pb2.DataSource.ValueType] | None = ...,
        services_list: abc.Iterable[_common_pb2.Service.ValueType] | None = ...,
        traffic_type: _common_pb2.TrafficType.ValueType = ...,
        only_restricted: builtins.bool | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = typing.Literal["_only_restricted", b"_only_restricted", "only_restricted", b"only_restricted"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["_only_restricted", b"_only_restricted", "only_restricted", b"only_restricted", "services_list", b"services_list", "sources_list", b"sources_list", "traffic_type", b"traffic_type"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType__only_restricted: _TypeAlias = typing.Literal["only_restricted"]  # noqa: Y015
    _WhichOneofArgType__only_restricted: _TypeAlias = typing.Literal["_only_restricted", b"_only_restricted"]  # noqa: Y015
    def WhichOneof(self, oneof_group: _WhichOneofArgType__only_restricted) -> _WhichOneofReturnType__only_restricted | None: ...

@typing.final
class AirportFilter(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    IATA_FIELD_NUMBER: builtins.int
    COUNTRY_ID_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    iata: builtins.str
    country_id: builtins.int
    type: AirportFilterType.ValueType
    def __init__(
        self,
        *,
        iata: builtins.str = ...,
        country_id: builtins.int = ...,
        type: AirportFilterType.ValueType = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["country_id", b"country_id", "iata", b"iata", "type", b"type"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class Interval(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    MIN_FIELD_NUMBER: builtins.int
    MAX_FIELD_NUMBER: builtins.int
    min: builtins.int
    max: builtins.int
    def __init__(
        self,
        *,
        min: builtins.int = ...,
        max: builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["max", b"max", "min", b"min"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class AirlineFilter(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    ICAO_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    icao: builtins.str
    """ICAO airline designator, e.g. `CPA`"""
    type: AirlineFilterType.ValueType
    def __init__(
        self,
        *,
        icao: builtins.str = ...,
        type: AirlineFilterType.ValueType = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["icao", b"icao", "type", b"type"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class Filter(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    ALTITUDE_RANGES_LIST_FIELD_NUMBER: builtins.int
    SPEED_RANGES_LIST_FIELD_NUMBER: builtins.int
    AIRLINES_LIST_FIELD_NUMBER: builtins.int
    CALLSIGNS_LIST_FIELD_NUMBER: builtins.int
    RADARS_LIST_FIELD_NUMBER: builtins.int
    REGS_LIST_FIELD_NUMBER: builtins.int
    AIRPORTS_LIST_FIELD_NUMBER: builtins.int
    FLIGHTS_LIST_FIELD_NUMBER: builtins.int
    TYPES_LIST_FIELD_NUMBER: builtins.int
    BIRTH_YEAR_RANGES_LIST_FIELD_NUMBER: builtins.int
    SQUAWKS_LIST_FIELD_NUMBER: builtins.int
    ORIGINS_LIST_FIELD_NUMBER: builtins.int
    DESTINATIONS_LIST_FIELD_NUMBER: builtins.int
    CATEGORIES_LIST_FIELD_NUMBER: builtins.int
    AIRSPACES_LIST_FIELD_NUMBER: builtins.int
    @builtins.property
    def altitude_ranges_list(self) -> containers.RepeatedCompositeFieldContainer[Interval]: ...
    @builtins.property
    def speed_ranges_list(self) -> containers.RepeatedCompositeFieldContainer[Interval]:
        """Minimum and maximum ground speed, knots"""

    @builtins.property
    def airlines_list(self) -> containers.RepeatedCompositeFieldContainer[AirlineFilter]: ...
    @builtins.property
    def callsigns_list(self) -> containers.RepeatedScalarFieldContainer[builtins.str]: ...
    @builtins.property
    def radars_list(self) -> containers.RepeatedScalarFieldContainer[builtins.str]:
        """List of receiver IDs, e.g. `["T-VHST000"]`"""

    @builtins.property
    def regs_list(self) -> containers.RepeatedScalarFieldContainer[builtins.str]: ...
    @builtins.property
    def airports_list(self) -> containers.RepeatedCompositeFieldContainer[AirportFilter]: ...
    @builtins.property
    def flights_list(self) -> containers.RepeatedScalarFieldContainer[builtins.str]: ...
    @builtins.property
    def types_list(self) -> containers.RepeatedScalarFieldContainer[builtins.str]:
        """List of aircraft types, e.g. `["B74*"]`"""

    @builtins.property
    def birth_year_ranges_list(self) -> containers.RepeatedCompositeFieldContainer[Interval]: ...
    @builtins.property
    def squawks_list(self) -> containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @builtins.property
    def origins_list(self) -> containers.RepeatedCompositeFieldContainer[AirportFilter]: ...
    @builtins.property
    def destinations_list(self) -> containers.RepeatedCompositeFieldContainer[AirportFilter]: ...
    @builtins.property
    def categories_list(self) -> containers.RepeatedScalarFieldContainer[_common_pb2.Service.ValueType]: ...
    @builtins.property
    def airspaces_list(self) -> containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        altitude_ranges_list: abc.Iterable[Interval] | None = ...,
        speed_ranges_list: abc.Iterable[Interval] | None = ...,
        airlines_list: abc.Iterable[AirlineFilter] | None = ...,
        callsigns_list: abc.Iterable[builtins.str] | None = ...,
        radars_list: abc.Iterable[builtins.str] | None = ...,
        regs_list: abc.Iterable[builtins.str] | None = ...,
        airports_list: abc.Iterable[AirportFilter] | None = ...,
        flights_list: abc.Iterable[builtins.str] | None = ...,
        types_list: abc.Iterable[builtins.str] | None = ...,
        birth_year_ranges_list: abc.Iterable[Interval] | None = ...,
        squawks_list: abc.Iterable[builtins.int] | None = ...,
        origins_list: abc.Iterable[AirportFilter] | None = ...,
        destinations_list: abc.Iterable[AirportFilter] | None = ...,
        categories_list: abc.Iterable[_common_pb2.Service.ValueType] | None = ...,
        airspaces_list: abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["airlines_list", b"airlines_list", "airports_list", b"airports_list", "airspaces_list", b"airspaces_list", "altitude_ranges_list", b"altitude_ranges_list", "birth_year_ranges_list", b"birth_year_ranges_list", "callsigns_list", b"callsigns_list", "categories_list", b"categories_list", "destinations_list", b"destinations_list", "flights_list", b"flights_list", "origins_list", b"origins_list", "radars_list", b"radars_list", "regs_list", b"regs_list", "speed_ranges_list", b"speed_ranges_list", "squawks_list", b"squawks_list", "types_list", b"types_list"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class LiveFeedRequest(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    BOUNDS_FIELD_NUMBER: builtins.int
    SETTINGS_FIELD_NUMBER: builtins.int
    FILTERS_LIST_FIELD_NUMBER: builtins.int
    FLEETS_LIST_FIELD_NUMBER: builtins.int
    HIGHLIGHT_MODE_FIELD_NUMBER: builtins.int
    STATS_FIELD_NUMBER: builtins.int
    LIMIT_FIELD_NUMBER: builtins.int
    MAXAGE_FIELD_NUMBER: builtins.int
    RESTRICTION_MODE_FIELD_NUMBER: builtins.int
    FIELD_MASK_FIELD_NUMBER: builtins.int
    SELECTED_FLIGHT_IDS_LIST_FIELD_NUMBER: builtins.int
    fleets_list: builtins.str
    """requires authentication"""
    highlight_mode: builtins.bool
    stats: builtins.bool
    """Include statistics for the given area"""
    limit: builtins.int
    """Maximum number of flights (default 1500 for unauthenticated users, 2000 for authenticated users)"""
    maxage: builtins.int
    """Maximum age since last update, seconds (default 14400)"""
    restriction_mode: _common_pb2.RestrictionVisibility.ValueType
    @builtins.property
    def bounds(self) -> LocationBoundaries: ...
    @builtins.property
    def settings(self) -> VisibilitySettings: ...
    @builtins.property
    def filters_list(self) -> Filter: ...
    @builtins.property
    def field_mask(self) -> field_mask_pb2.FieldMask: ...
    @builtins.property
    def selected_flight_ids_list(self) -> containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        bounds: LocationBoundaries | None = ...,
        settings: VisibilitySettings | None = ...,
        filters_list: Filter | None = ...,
        fleets_list: builtins.str = ...,
        highlight_mode: builtins.bool = ...,
        stats: builtins.bool | None = ...,
        limit: builtins.int | None = ...,
        maxage: builtins.int | None = ...,
        restriction_mode: _common_pb2.RestrictionVisibility.ValueType | None = ...,
        field_mask: field_mask_pb2.FieldMask | None = ...,
        selected_flight_ids_list: abc.Iterable[builtins.int] | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = typing.Literal["_limit", b"_limit", "_maxage", b"_maxage", "_restriction_mode", b"_restriction_mode", "_stats", b"_stats", "bounds", b"bounds", "field_mask", b"field_mask", "filters_list", b"filters_list", "limit", b"limit", "maxage", b"maxage", "restriction_mode", b"restriction_mode", "settings", b"settings", "stats", b"stats"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["_limit", b"_limit", "_maxage", b"_maxage", "_restriction_mode", b"_restriction_mode", "_stats", b"_stats", "bounds", b"bounds", "field_mask", b"field_mask", "filters_list", b"filters_list", "fleets_list", b"fleets_list", "highlight_mode", b"highlight_mode", "limit", b"limit", "maxage", b"maxage", "restriction_mode", b"restriction_mode", "selected_flight_ids_list", b"selected_flight_ids_list", "settings", b"settings", "stats", b"stats"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType__limit: _TypeAlias = typing.Literal["limit"]  # noqa: Y015
    _WhichOneofArgType__limit: _TypeAlias = typing.Literal["_limit", b"_limit"]  # noqa: Y015
    _WhichOneofReturnType__maxage: _TypeAlias = typing.Literal["maxage"]  # noqa: Y015
    _WhichOneofArgType__maxage: _TypeAlias = typing.Literal["_maxage", b"_maxage"]  # noqa: Y015
    _WhichOneofReturnType__restriction_mode: _TypeAlias = typing.Literal["restriction_mode"]  # noqa: Y015
    _WhichOneofArgType__restriction_mode: _TypeAlias = typing.Literal["_restriction_mode", b"_restriction_mode"]  # noqa: Y015
    _WhichOneofReturnType__stats: _TypeAlias = typing.Literal["stats"]  # noqa: Y015
    _WhichOneofArgType__stats: _TypeAlias = typing.Literal["_stats", b"_stats"]  # noqa: Y015
    @typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__limit) -> _WhichOneofReturnType__limit | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__maxage) -> _WhichOneofReturnType__maxage | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__restriction_mode) -> _WhichOneofReturnType__restriction_mode | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: _WhichOneofArgType__stats) -> _WhichOneofReturnType__stats | None: ...

@typing.final
class LiveFeedResponse(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    FLIGHTS_LIST_FIELD_NUMBER: builtins.int
    STATS_FIELD_NUMBER: builtins.int
    SELECTED_FLIGHT_LIST_FIELD_NUMBER: builtins.int
    SERVER_TIME_MS_FIELD_NUMBER: builtins.int
    server_time_ms: builtins.int
    @builtins.property
    def flights_list(self) -> containers.RepeatedCompositeFieldContainer[_common_pb2.Flight]: ...
    @builtins.property
    def stats(self) -> _common_pb2.Stats: ...
    @builtins.property
    def selected_flight_list(self) -> containers.RepeatedCompositeFieldContainer[_common_pb2.Flight]: ...
    def __init__(
        self,
        *,
        flights_list: abc.Iterable[_common_pb2.Flight] | None = ...,
        stats: _common_pb2.Stats | None = ...,
        selected_flight_list: abc.Iterable[_common_pb2.Flight] | None = ...,
        server_time_ms: builtins.int = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = typing.Literal["stats", b"stats"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["flights_list", b"flights_list", "selected_flight_list", b"selected_flight_list", "server_time_ms", b"server_time_ms", "stats", b"stats"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class PlaybackRequest(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    LIVE_FEED_REQUEST_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    PREFETCH_FIELD_NUMBER: builtins.int
    HFREQ_FIELD_NUMBER: builtins.int
    timestamp: builtins.int
    """Start timestamp"""
    prefetch: builtins.int
    """End timestamp, should be timestamp + floor(7.5*(multiplier)) seconds

    For 1x playback, it should be timestamp + 7 seconds.
    """
    hfreq: builtins.int
    """High frequency mode, likely used to return granular data (?)"""
    @builtins.property
    def live_feed_request(self) -> LiveFeedRequest: ...
    def __init__(
        self,
        *,
        live_feed_request: LiveFeedRequest | None = ...,
        timestamp: builtins.int = ...,
        prefetch: builtins.int = ...,
        hfreq: builtins.int | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = typing.Literal["_hfreq", b"_hfreq", "hfreq", b"hfreq", "live_feed_request", b"live_feed_request"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["_hfreq", b"_hfreq", "hfreq", b"hfreq", "live_feed_request", b"live_feed_request", "prefetch", b"prefetch", "timestamp", b"timestamp"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType__hfreq: _TypeAlias = typing.Literal["hfreq"]  # noqa: Y015
    _WhichOneofArgType__hfreq: _TypeAlias = typing.Literal["_hfreq", b"_hfreq"]  # noqa: Y015
    def WhichOneof(self, oneof_group: _WhichOneofArgType__hfreq) -> _WhichOneofReturnType__hfreq | None: ...

@typing.final
class PlaybackResponse(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    LIVE_FEED_RESPONSE_FIELD_NUMBER: builtins.int
    @builtins.property
    def live_feed_response(self) -> LiveFeedResponse:
        """NOTE: position buffer is not included in the response."""

    def __init__(
        self,
        *,
        live_feed_response: LiveFeedResponse | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = typing.Literal["live_feed_response", b"live_feed_response"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["live_feed_response", b"live_feed_response"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...



from __future__ import annotations

from typing import Literal, Union

from typing_extensions import (
    Annotated,
    NotRequired,
    Required,
    TypedDict,
    TypeGuard,
)

from . import StrFlightIdHex
from .isqx import (
    AltitudeFt,
    AltitudeM,
    BankAngleDeg,
    BearingDeg,
    DurationS,
    GeometricAltitudeM,
    GroundSpeedKmh,
    GroundSpeedKt,
    GroundSpeedMph,
    GroundTrackDeg,
    IndicatedAirSpeedKt,
    LatitudeDeg,
    LongitudeDeg,
    MachTimes1K,
    PressureAltimeterHpa,
    StaticTemperatureC,
    TimestampS,
    TrueAirSpeedKt,
    UtcOffsetS,
    VerticalSpeedFpm,
    VerticalSpeedMps,
    WindSpeedKt,
)

#
# authentication
#


class User(TypedDict, total=False):
    id: Required[int]
    identity: str
    locale: str


class Features(TypedDict): ...  # useless anyway


class UserData(TypedDict, total=False):
    accessToken: None | str
    accountType: str
    countryCode: None | str
    dateExpires: TimestampS[int]
    dateLastLogin: str
    features: Features
    hasConsented: bool
    hasPassword: bool
    idUser: int
    identity: str
    isActive: bool
    isAnonymousAccount: bool
    isLoggedIn: bool
    localeCode: str
    name: None
    oAuth: None
    oAuthType: None
    publicKey: None
    subscriptionKey: None | str
    tokenLogin: str
    typeSource: str


class Authentication(TypedDict, total=False):
    message: str
    msg: str
    response_code: int
    responseCode: int
    status: str
    success: bool
    token: str
    user: User
    features: Features
    userData: Required[UserData]


class UsernamePassword(TypedDict):
    username: str
    password: str


class TokenSubscriptionKey(TypedDict):
    subscriptionKey: str
    token: str


#
# common
#


class APIResult(TypedDict):
    copyright: str
    legalNotice: str


class FlightNumber(TypedDict):
    default: str | None
    alternative: NotRequired[str | None]


class AircraftModel(TypedDict):
    code: str
    text: None | str


class AircraftAge(TypedDict):
    availability: bool
    date: NotRequired[str]  # if unauthenticated


class AircraftAvailability(TypedDict):
    serialNo: bool
    age: bool


class _GenericStatus(TypedDict):
    text: str | None
    type: str
    color: str | None
    diverted: str | None


class _GenericEventTime(TypedDict):
    utc: TimestampS[int] | None
    local: TimestampS[int] | None


class _StatusGeneric(TypedDict):
    status: _GenericStatus
    eventTime: _GenericEventTime


class StatusData(TypedDict):
    live: bool | None
    text: str | None
    icon: str | None
    estimated: None
    ambiguous: bool
    generic: _StatusGeneric


class _Iata_Icao(TypedDict):
    iata: str | None
    icao: str


class OwnerData(TypedDict):
    name: str
    code: _Iata_Icao


class AirlineData(TypedDict):
    name: str
    code: _Iata_Icao
    short: str


class _AirportCode(TypedDict):
    iata: str
    icao: str


class _Country(TypedDict):
    name: str
    code: str
    id: int


class _Region(TypedDict):
    city: str


class _AirportPosition(TypedDict):
    latitude: LatitudeDeg[float]
    longitude: LongitudeDeg[float]
    country: _Country
    region: _Region


class _Timezone(TypedDict):
    name: str
    offset: UtcOffsetS[int]
    abbr: str
    abbrName: str | None
    isDst: bool


# prior to v0.2.0 this was `common.Airport`
class CommonAirport(TypedDict):
    name: str
    code: _AirportCode
    position: _AirportPosition
    timezone: _Timezone
    visible: NotRequired[bool]


# O/D can be null when route status is "unknown"
class AirportPairData(TypedDict):
    origin: None | CommonAirport
    destination: None | CommonAirport
    real: None | CommonAirport
    """Destination airport for diverted"""


class _Image(TypedDict):
    src: str
    link: str
    copyright: str
    source: str


class ImageCollection(TypedDict):
    large: list[_Image]
    medium: list[_Image]
    thumbnails: list[_Image]


#
# flight list
#


class FlightListRequest(TypedDict, total=False):
    callback: None
    device: None | str
    fetchBy: Required[str]
    filterBy: str | None
    format: Literal["json"]
    limit: Required[int]
    olderThenFlightId: None
    page: int
    pk: None
    query: Required[str]
    timestamp: TimestampS[int]
    token: None | str


class Item(TypedDict):
    current: int
    total: None | int
    limit: int


class Page(TypedDict):
    current: int
    more: bool
    total: None | int


class Identification(TypedDict):
    id: str | None
    """icao hex"""
    row: int
    """internal row id"""
    number: FlightNumber
    callsign: str | None
    codeshare: None


class FlightListCountry(TypedDict):
    id: int | None
    name: str | None
    alpha2: str | None
    alpha3: str | None


class AircraftInfo(TypedDict):
    model: AircraftModel
    registration: None | str
    owner: NotRequired[OwnerData]
    airline: NotRequired[AirlineData]
    country: None | FlightListCountry
    hex: None | str
    restricted: bool
    serialNo: None | str
    age: AircraftAge
    availability: AircraftAvailability


class Interval(TypedDict):
    departure: None | TimestampS[int]
    arrival: None | TimestampS[int]


class TimeOther(TypedDict):
    eta: None | TimestampS[int]
    updated: None | TimestampS[int]
    duration: None | DurationS[int]


class FlightListTime(TypedDict):
    scheduled: Interval
    real: Interval
    estimated: Interval
    other: TimeOther


class FlightListItem(TypedDict):
    identification: Identification
    status: StatusData
    aircraft: AircraftInfo
    owner: OwnerData
    airline: AirlineData
    airport: AirportPairData
    time: FlightListTime


class AircraftImage(TypedDict):
    registration: str
    images: ImageCollection


class FlightListResponse(TypedDict):
    item: Item
    page: Page
    timestamp: TimestampS[int]
    data: list[FlightListItem] | None
    # TODO: depending on whether the flight list was queried by reg or flight
    # this may change - check this again
    aircraftInfo: AircraftInfo
    aircraftImages: list[AircraftImage]


class FlightListResult(TypedDict):
    request: FlightListRequest
    response: FlightListResponse


class FlightList(TypedDict):
    result: FlightListResult
    _api: APIResult


FLIGHT_LIST_EMPTY: FlightList = {
    "result": {
        "request": {
            "fetchBy": "flight",
            "format": "json",
            "limit": 0,
            "page": 0,
            "query": "",
            "timestamp": 0,
        },
        "response": {
            "item": {"current": 0, "total": None, "limit": 0},
            "page": {"current": 0, "more": False, "total": None},
            "timestamp": 0,
            "data": None,
            "aircraftInfo": {
                "model": {"code": "", "text": ""},
                "registration": None,
                "country": None,
                "hex": None,
                "restricted": False,
                "serialNo": None,
                "age": {"availability": False},
                "availability": {"serialNo": False, "age": False},
            },
            "aircraftImages": [],
        },
    },
    "_api": {"copyright": "", "legalNotice": ""},
}
"""An object with no flight list data, for use as a default value."""

#
# playback types
#


class PlaybackRequest(TypedDict, total=False):
    callback: None
    device: None | str
    flightId: Required[StrFlightIdHex]
    format: Literal["json"]
    pk: None
    timestamp: TimestampS[int] | None
    token: None | str


class FlightIdentification(TypedDict):
    id: StrFlightIdHex | int
    number: FlightNumber
    callsign: str


class AircraftIdentification(TypedDict):
    modes: str
    registration: str
    serialNo: None | str
    age: NotRequired[AircraftAge]  # if unauthenticated
    availability: NotRequired[AircraftAvailability]  # if unauthenticated


class AircraftData(TypedDict):
    model: AircraftModel
    identification: AircraftIdentification
    availability: AircraftAvailability


class Median(TypedDict):
    time: int | None  # TODO: unit?
    delay: int | None  # TOD: unit?
    timestamp: TimestampS[int] | None


class Altitude(TypedDict):
    feet: AltitudeFt[int]
    meters: AltitudeM[int]


class Speed(TypedDict):
    kmh: GroundSpeedKmh[float]
    kts: GroundSpeedKt[int]
    mph: GroundSpeedMph[float]


class VerticalSpeed(TypedDict):
    fpm: VerticalSpeedFpm[int] | None
    ms: VerticalSpeedMps[int] | None


class EMS(TypedDict):
    ts: TimestampS[int]
    ias: IndicatedAirSpeedKt[int] | None
    tas: TrueAirSpeedKt[int] | None
    mach: MachTimes1K[int] | None
    mcp: AltitudeFt[int] | None
    fms: AltitudeFt[int] | None
    autopilot: None
    oat: StaticTemperatureC[int] | None
    trueTrack: GroundTrackDeg[float] | None
    rollAngle: BankAngleDeg[float] | None
    qnh: PressureAltimeterHpa[int] | None
    windDir: BearingDeg[int] | None
    windSpd: WindSpeedKt[int] | None
    precision: int | None
    altGPS: GeometricAltitudeM[int] | None
    emergencyStatus: int | None
    tcasAcasDtatus: int | None
    heading: BearingDeg[int] | None


class TrackData(TypedDict):
    latitude: LatitudeDeg[float]
    longitude: LongitudeDeg[float]
    altitude: Altitude
    speed: Speed
    verticalSpeed: VerticalSpeed
    heading: GroundTrackDeg[int]
    """
    !!! warning
        The JSON response claims that `heading` is available, but ADS-B only
        transmits the [ground track](https://mode-s.org/decode/content/ads-b/4-surface-position.html#ground-track).
        [Heading](https://mode-s.org/decode/content/mode-s/7-ehs.html#heading-and-speed-report-bds-60)
        is only available in [EMS][fr24.types.json.EMS] data.

        This field is renamed to `track` to avoid confusion in
        [fr24.json.playback_track_dict][].
    """
    squawk: str
    timestamp: TimestampS[int]
    ems: None | EMS


class FlightDataAvailability(TypedDict):
    ems: bool


class FlightData(TypedDict):
    identification: FlightIdentification
    status: StatusData
    aircraft: AircraftData | None
    owner: OwnerData | None
    airline: AirlineData | None
    airport: AirportPairData
    median: Median
    track: list[TrackData]
    aircraftImages: ImageCollection
    availability: FlightDataAvailability


class PlaybackData(TypedDict):
    flight: FlightData


class PlaybackResponse(TypedDict):
    timestamp: TimestampS[int]
    altitudeFiltered: bool
    data: PlaybackData


class PlaybackResult(TypedDict):
    request: PlaybackRequest
    response: PlaybackResponse


class Playback(TypedDict):
    result: PlaybackResult
    _api: APIResult


#
# airport list
#


# prior to v0.2.0 this was `airport_list.Schedule`
class AirportListScheduleSetting(TypedDict):
    mode: Literal["departures", "arrivals"] | None
    timestamp: TimestampS[int]


Plugin = Literal[
    "details",
    "runways",
    "schedule",
    "satelliteImage",
    "scheduledRoutesStatistics",
    "weather",
]


class PluginSetting(TypedDict, total=False):
    schedule: AirportListScheduleSetting


AirportRequest = TypedDict(
    # we need this syntax because of the plugin-setting parameter
    "AirportRequest",
    {
        "callback": None,
        "code": str,
        "device": Union[str, None],
        "fleet": Union[str, None],
        "format": Literal["json"],
        "limit": int,
        "page": int,
        "pk": None,
        "plugin": list[Plugin],
        "plugin[]": list[Plugin],
        "plugin-setting": PluginSetting,
        "plugin-setting[schedule][mode]": str,
        "plugin-setting[schedule][timestamp]": TimestampS[int],
        "token": Union[str, None],
    },
    total=False,
)


class AirportSchedule(TypedDict):
    arrivals: FlightListResponse
    departures: FlightListResponse


class AirportPluginData(TypedDict):
    schedule: AirportSchedule


class AirportListData(TypedDict):
    pluginData: AirportPluginData


class AirportResponse(TypedDict):
    airport: AirportListData


class AirportResult(TypedDict):
    request: AirportRequest
    response: AirportResponse


class AirportList(TypedDict):
    result: AirportResult
    _api: APIResult


#
# find
#


class Live(TypedDict):
    operator_id: NotRequired[int]
    lat: LatitudeDeg[float]
    lon: LongitudeDeg[float]
    schd_from: NotRequired[str]
    schd_to: NotRequired[str]
    ac_type: str
    route: NotRequired[str]
    logo: NotRequired[str]
    reg: str
    callsign: NotRequired[str]
    flight: NotRequired[str]
    operator: NotRequired[str]


class FindAirportDetail(TypedDict):
    lat: LatitudeDeg[float]
    lon: LongitudeDeg[float]
    size: float  # TODO: unit?


class Operator(TypedDict):
    operator_id: int
    iata: NotRequired[str]
    logo: NotRequired[str]


class FindScheduleDetail(TypedDict):
    logo: NotRequired[str]
    callsign: NotRequired[str]
    flight: str
    operator: NotRequired[str]
    operator_id: NotRequired[int]
    schd_from: NotRequired[str]
    schd_to: NotRequired[str]


class Aircraft(TypedDict):
    owner: str  # icao
    equip: str  # aircraft type
    hex: str
    operator_id: int
    logo: str


class EntryBase(TypedDict):
    label: str
    name: NotRequired[str]


Iata = Annotated[str, "IATA"]
Icao = Annotated[str, "ICAO"]


class AirportEntry(EntryBase):
    id: Iata | Icao  # iata if match == "begins"
    detail: FindAirportDetail
    type: Literal["airport"]
    match: Literal["icao", "iata", "begins", "contains"]


class OperatorEntry(EntryBase):
    id: str
    detail: Operator
    type: Literal["operator"]
    match: Literal["begins", "icao"]


class LiveEntry(EntryBase):
    id: str
    detail: Live
    type: Literal["live"]
    match: Literal["route", "begins"]


# prior to v0.2.0 this was `find.Schedule`
class ScheduleEntry(EntryBase):
    id: str
    detail: FindScheduleDetail
    type: Literal["schedule"]
    match: Literal["route", "begins"]


class AircraftEntry(EntryBase):
    id: str
    detail: Aircraft
    type: Literal["aircraft"]
    match: Literal["begins"]


Entry = Union[
    AirportEntry, OperatorEntry, LiveEntry, ScheduleEntry, AircraftEntry
]
# NOTE: in tests, we use Annotated[Entry, pydantic.Discriminator("type")]
# not adding here because pydantic belongs to test dependencies


def is_airport(entry: Entry) -> TypeGuard[AirportEntry]:
    return entry["type"] == "airport"


def is_operator(entry: Entry) -> TypeGuard[OperatorEntry]:
    return entry["type"] == "operator"


def is_live(entry: Entry) -> TypeGuard[LiveEntry]:
    return entry["type"] == "live"


def is_schedule(entry: Entry) -> TypeGuard[ScheduleEntry]:
    return entry["type"] == "schedule"


def is_aircraft(entry: Entry) -> TypeGuard[AircraftEntry]:
    return entry["type"] == "aircraft"


class StatsEntry(TypedDict):
    all: NotRequired[int]
    airport: int
    operator: int
    live: int
    schedule: int
    aircraft: int


class Stats(TypedDict):
    total: StatsEntry
    count: StatsEntry


class Info(TypedDict):
    grpcEnabled: bool


class Find(TypedDict):
    results: list[Entry]
    stats: Stats
    info: Info



# ruff: noqa
# mypy: disable-error-code="top-level-await, no-redef"
# %%
# --8<-- [start:script0]
import httpx
from fr24.grpc import follow_flight_stream
from fr24.proto.v1_pb2 import FollowFlightRequest, FollowFlightResponse
from fr24.proto import parse_data
from fr24.proto.headers import get_grpc_headers


async def follow_flight_data() -> None:
    timeout = httpx.Timeout(5, read=120)
    headers = httpx.Headers(get_grpc_headers(auth=None))
    async with httpx.AsyncClient(timeout=timeout) as client:
        message = FollowFlightRequest(flight_id=0x3C500FDB)
        i = 0
        async for response in follow_flight_stream(client, message, headers):
            print(f"##### {i} #####")
            print(parse_data(response, FollowFlightResponse))
            i += 1
            if i > 3:
                break


await follow_flight_data()
# --8<-- [end:script0]
# %%
"""
# --8<-- [start:output0]
##### 0 #####
Ok(_value=aircraft_info {
  icao_address: 7866977
  reg: "B-KQM"
  type: "B77W"
  icon: B777
  full_description: "Boeing 777-367(ER)"
  images_list {
    url: "https://www.jetphotos.com/photo/11825817"
    copyright: "ZSHC_Linzx"
    thumbnail: "https://cdn.jetphotos.com/200/5/1291679_1756473618_tb.jpg"
    medium: "https://cdn.jetphotos.com/400/5/1291679_1756473618.jpg"
    large: "https://cdn.jetphotos.com/640/5/1291679_1756473618.jpg"
  }
  images_list {
    url: "https://www.jetphotos.com/photo/11813969"
    copyright: "CAN-Eric"
    thumbnail: "https://cdn.jetphotos.com/200/5/967682_1755532162_tb.jpg"
    medium: "https://cdn.jetphotos.com/400/5/967682_1755532162.jpg"
    large: "https://cdn.jetphotos.com/640/5/967682_1755532162.jpg"
  }
  images_list {
    url: "https://www.jetphotos.com/photo/11803561"
    copyright: "cc12214"
    thumbnail: "https://cdn.jetphotos.com/200/6/585221_1754572527_tb.jpg"
    medium: "https://cdn.jetphotos.com/400/6/585221_1754572527.jpg"
    large: "https://cdn.jetphotos.com/640/6/585221_1754572527.jpg"
  }
  msn_available: true
  age_available: true
  registered_owners: "Cathay Pacific (Oneworld Livery)"
  is_country_of_reg_available: true
}
schedule_info {
  flight_number: "CX251"
  operated_by_id: 57
  painted_as_id: 57
  origin_id: 1366
  destination_id: 1942
  scheduled_departure: 1758465600
  scheduled_arrival: 1758516000
  actual_departure: 1758466901
  arr_terminal: "3"
}
flight_progress {
  traversed_distance: 21783
  remaining_distance: 9668300
  elapsed_time: 295
  remaining_time: 51569
  eta: 1758518765
  great_circle_distance: 9647791
  mean_flight_time: 48161
  flight_stage: ASCENDING
  delay_status: RED
}
flight_info {
  flightid: 1011879899
  lat: 22.1949
  lon: 114.087921
  track: 169
  alt: 7525
  speed: 303
  timestamp_ms: 1758467194561
  callsign: "CPA251"
  ems_availability {
    qnh_availability: true
    amcp_availability: true
    oat_availability: true
    ias_availability: true
    tas_availability: true
    mach_availability: true
    agps_availability: true
    agpsdiff_availability: true
    wind_dir_availability: true
    wind_speed_availability: true
    rs_availability: true
  }
  squawk_availability: true
  vspeed_availability: true
  airspace_availability: true
  server_time_ms: 1758467196527
}
flight_trail_list {
  snapshot_id: 1758465991
  lat: 22.3132782
  lon: 113.934616
  heading: 70
}
flight_trail_list {
  snapshot_id: 1758466089
  lat: 22.3131981
  lon: 113.934303
  spd: 2
  heading: 70
}
...
flight_trail_list {
  snapshot_id: 1758467188
  lat: 22.2043304
  lon: 114.086678
  altitude: 7450
  spd: 300
  heading: 179
}
position_buffer {
  recent_positions_list {
    delta_lat: -155
    delta_lon: 34
    delta_ms: 1090
  }
  recent_positions_list {
    delta_lat: -517
    delta_lon: 94
    delta_ms: 3710
  }
  recent_positions_list {
    delta_lat: -746
    delta_lon: 119
    delta_ms: 5270
  }
  recent_positions_list {
    delta_lat: -942
    delta_lon: 124
    delta_ms: 6700
  }
  recent_positions_list {
    delta_lat: -1107
    delta_lon: 129
    delta_ms: 7870
  }
  recent_positions_list {
    delta_lat: -1364
    delta_lon: 129
    delta_ms: 9710
  }
  recent_positions_list {
    delta_lat: -1597
    delta_lon: 129
    delta_ms: 11380
  }
}
)
##### 1 #####
Ok(_value=schedule_info {
  flight_number: "CX251"
  operated_by_id: 57
  painted_as_id: 57
  origin_id: 1366
  destination_id: 1942
  scheduled_departure: 1758465600
  scheduled_arrival: 1758516000
  actual_departure: 1758466901
  arr_terminal: "3"
}
flight_progress {
  traversed_distance: 21940
  remaining_distance: 9668494
  elapsed_time: 298
  remaining_time: 51566
  eta: 1758518765
  great_circle_distance: 9647791
  mean_flight_time: 48161
  flight_stage: ASCENDING
  delay_status: RED
}
flight_info {
  flightid: 1011879899
  lat: 22.1930809
  lon: 114.088387
  track: 167
  alt: 7550
  speed: 303
  timestamp_ms: 1758467195921
  callsign: "CPA251"
  ems_availability {
    qnh_availability: true
    amcp_availability: true
    oat_availability: true
    ias_availability: true
    tas_availability: true
    mach_availability: true
    agps_availability: true
    agpsdiff_availability: true
    wind_dir_availability: true
    wind_speed_availability: true
    rs_availability: true
  }
  squawk_availability: true
  vspeed_availability: true
  airspace_availability: true
  server_time_ms: 1758467199530
}
)
##### 2 #####
Ok(_value=schedule_info {
  flight_number: "CX251"
  operated_by_id: 57
  painted_as_id: 57
  origin_id: 1366
  destination_id: 1942
  scheduled_departure: 1758465600
  scheduled_arrival: 1758516000
  actual_departure: 1758466901
  arr_terminal: "3"
}
flight_progress {
  traversed_distance: 22437
  remaining_distance: 9669070
  elapsed_time: 301
  remaining_time: 51563
  eta: 1758518765
  great_circle_distance: 9647791
  mean_flight_time: 48161
  flight_stage: ASCENDING
  delay_status: RED
}
flight_info {
  flightid: 1011879899
  lat: 22.1880341
  lon: 114.09037
  track: 158
  alt: 7600
  speed: 305
  timestamp_ms: 1758467199681
  callsign: "CPA251"
  ems_availability {
    qnh_availability: true
    amcp_availability: true
    oat_availability: true
    ias_availability: true
    tas_availability: true
    mach_availability: true
    agps_availability: true
    agpsdiff_availability: true
    wind_dir_availability: true
    wind_speed_availability: true
    rs_availability: true
  }
  squawk_availability: true
  vspeed_availability: true
  airspace_availability: true
  server_time_ms: 1758467202532
}
)
##### 3 #####
Ok(_value=schedule_info {
  flight_number: "CX251"
  operated_by_id: 57
  painted_as_id: 57
  origin_id: 1366
  destination_id: 1942
  scheduled_departure: 1758465600
  scheduled_arrival: 1758516000
  actual_departure: 1758466901
  arr_terminal: "3"
}
flight_progress {
  traversed_distance: 22657
  remaining_distance: 9669316
  elapsed_time: 304
  remaining_time: 51560
  eta: 1758518765
  great_circle_distance: 9647791
  mean_flight_time: 48161
  flight_stage: ASCENDING
  delay_status: RED
}
flight_info {
  flightid: 1011879899
  lat: 22.1859741
  lon: 114.09137
  track: 154
  alt: 7625
  speed: 305
  timestamp_ms: 1758467201271
  callsign: "CPA251"
  ems_availability {
    qnh_availability: true
    amcp_availability: true
    oat_availability: true
    ias_availability: true
    tas_availability: true
    mach_availability: true
    agps_availability: true
    agpsdiff_availability: true
    wind_dir_availability: true
    wind_speed_availability: true
    rs_availability: true
  }
  squawk_availability: true
  vspeed_availability: true
  airspace_availability: true
  server_time_ms: 1758467205534
}
)
# --8<-- [end:output0]
"""
